# Copyright 2017-2022 Hewlett Packard Enterprise Development LP
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json
from unittest import mock

import testtools

from proliantutils.redfish.resources.system import pci_device


class PCIDeviceTestCase(testtools.TestCase):

    def setUp(self):
        super(PCIDeviceTestCase, self).setUp()
        self.conn = mock.Mock()
        pci_file = 'proliantutils/tests/redfish/json_samples/pci_device.json'
        with open(pci_file, 'r') as f:
            self.json_doc = json.load(f)
        self.conn.get.return_value.json.return_value = self.json_doc

        pci_path = "/redfish/v1/Systems/1/PCIDevices/1"
        self.sys_pci = pci_device.PCIDevice(
            self.conn, pci_path, redfish_version='1.0.2',
            registries=None, root=None)

    def test__parse_attributes(self):
        self.sys_pci._parse_attributes(self.json_doc)
        self.assertEqual('1.0.2', self.sys_pci.redfish_version)
        self.assertEqual('1', self.sys_pci.identity)
        self.assertEqual(1, self.sys_pci.nic_capacity)


class PCIDeviceCollectionTestCase(testtools.TestCase):

    def setUp(self):
        super(PCIDeviceCollectionTestCase, self).setUp()
        self.conn = mock.Mock()
        with open('proliantutils/tests/redfish/json_samples/'
                  'pci_device_collection.json') as f:
            self.json_doc = json.load(f)
        self.conn.get.return_value.json.return_value = self.json_doc
        self.sys_pci_col = pci_device.PCIDeviceCollection(
            self.conn, '/redfish/v1/Systems/1/PCIDevices',
            redfish_version='1.0.2', registries=None, root=None)

    def test__parse_attributes(self):
        self.sys_pci_col._parse_attributes(self.json_doc)
        self.assertEqual('1.0.2', self.sys_pci_col.redfish_version)
        self.assertEqual('PciDevices', self.sys_pci_col.name)
        pci_path = ('/redfish/v1/Systems/1/PCIDevices/1',
                    '/redfish/v1/Systems/1/PCIDevices/6')
        self.assertEqual(pci_path, self.sys_pci_col.members_identities)

    @mock.patch.object(pci_device, 'PCIDevice', autospec=True)
    def test_get_member(self, mock_pci):
        self.sys_pci_col.get_member(
            '/redfish/v1/Systems/1/PCIDevices/1')
        mock_pci.assert_called_once_with(
            self.sys_pci_col._conn,
            ('/redfish/v1/Systems/1/PCIDevices/1'),
            redfish_version=self.sys_pci_col.redfish_version,
            registries=None, root=self.sys_pci_col.root)

    @mock.patch.object(pci_device, 'PCIDevice', autospec=True)
    def test_get_members(self, mock_pci):
        members = self.sys_pci_col.get_members()
        path_list = ["/redfish/v1/Systems/1/PCIDevices/1",
                     "/redfish/v1/Systems/1/PCIDevices/6"]
        calls = [
            mock.call(self.sys_pci_col._conn, path_list[0],
                      redfish_version=self.sys_pci_col.redfish_version,
                      registries=None, root=self.sys_pci_col.root),
            mock.call(self.sys_pci_col._conn, path_list[1],
                      redfish_version=self.sys_pci_col.redfish_version,
                      registries=None, root=self.sys_pci_col.root)
        ]
        mock_pci.assert_has_calls(calls)
        self.assertIsInstance(members, list)
        self.assertEqual(2, len(members))

    def test_gpu_devices(self):
        self.conn.get.return_value.json.reset_mock()
        val = []
        path = ('proliantutils/tests/redfish/json_samples/'
                'pci_device.json')
        with open(path, 'r') as f:
            val.append(json.loads(f.read()))
        path = ('proliantutils/tests/redfish/json_samples/'
                'pci_device1.json')
        with open(path, 'r') as f:
            val.append(json.loads(f.read()))
        self.conn.get.return_value.json.side_effect = val
        expected = ["/redfish/v1/Systems/1/PCIDevices/6/"]
        actual = self.sys_pci_col.gpu_devices
        self.assertEqual(1, len(actual))
        self.assertEqual(expected, actual)

    def test_max_nic_capacity(self):
        self.conn.get.return_value.json.reset_mock()
        val = []
        path = ('proliantutils/tests/redfish/json_samples/'
                'pci_device.json')
        with open(path, 'r') as f:
            val.append(json.loads(f.read()))
        path = ('proliantutils/tests/redfish/json_samples/'
                'pci_device1.json')
        with open(path, 'r') as f:
            val.append(json.loads(f.read()))
        path = ('proliantutils/tests/redfish/json_samples/'
                'pci_device2.json')
        with open(path, 'r') as f:
            val.append(json.loads(f.read()))
        self.conn.get.return_value.json.side_effect = val
        actual_capacity = self.sys_pci_col.max_nic_capacity
        expected = '1Gb'
        self.assertEqual(expected, actual_capacity)

    def test_vendor_id(self):
        self.conn.get.return_value.json.reset_mock()
        val = []
        path = ('proliantutils/tests/redfish/json_samples/'
                'pci_device.json')
        with open(path, 'r') as f:
            val.append(json.loads(f.read()))
        path = ('proliantutils/tests/redfish/json_samples/'
                'pci_device1.json')
        with open(path, 'r') as f:
            val.append(json.loads(f.read()))
        self.conn.get.return_value.json.side_effect = val
        expected_vendor_id = {'6': 4139}
        self.assertEqual(expected_vendor_id,
                         self.sys_pci_col.vendor_id)



# Copyright 2017 Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
__author__ = 'HPE'

from unittest import mock

import ddt
import sushy
import testtools

from proliantutils import exception
from proliantutils.redfish.resources.system.storage import common


@ddt.ddt
class CommonMethodsTestCase(testtools.TestCase):

    def setUp(self):
        super(CommonMethodsTestCase, self).setUp()
        self.system_obj = mock.MagicMock()

    def _mock_property(self, value):
        if value is sushy.exceptions.SushyError:
            mock_value = mock.PropertyMock(side_effect=value)
        else:
            mock_value = mock.PropertyMock(return_value=value)
        return mock_value

    @ddt.data((953837, 60000, 60000, 60000, 60000, 930),
              (sushy.exceptions.SushyError, 1000169537536, 60000, 60000,
               60000, 930),
              (953837, sushy.exceptions.SushyError, 60000, 60000, 60000,
               930),
              (sushy.exceptions.SushyError, sushy.exceptions.SushyError,
               953837, 60000, 40000, 930),
              (sushy.exceptions.SushyError, sushy.exceptions.SushyError,
               sushy.exceptions.SushyError, 1000169537536, 40000, 930),
              (sushy.exceptions.SushyError, sushy.exceptions.SushyError,
               sushy.exceptions.SushyError, sushy.exceptions.SushyError,
               1000169537536, 930),
              (sushy.exceptions.SushyError, sushy.exceptions.SushyError,
               sushy.exceptions.SushyError, sushy.exceptions.SushyError,
               sushy.exceptions.SushyError, 0),
              )
    @ddt.unpack
    def test_get_local_gb(self, logical_max, volume_max, physical_max,
                          drive_max, simple_max, expected):

        system_obj = self.system_obj
        type(system_obj.smart_storage).logical_drives_maximum_size_mib = (
            self._mock_property(logical_max))
        type(system_obj.storages).volumes_maximum_size_bytes = (
            self._mock_property(volume_max))
        type(system_obj.smart_storage).physical_drives_maximum_size_mib = (
            self._mock_property(physical_max))
        type(system_obj.storages).drives_maximum_size_bytes = (
            self._mock_property(drive_max))
        type(system_obj.simple_storages).maximum_size_bytes = (
            self._mock_property(simple_max))
        actual = common.get_local_gb(system_obj)
        self.assertEqual(expected, actual)

    def test__get_attribute_value_of(self):
        system_obj = self.system_obj
        si_mock = mock.PropertyMock(return_value=1000169537536)
        type(system_obj.simple_storages).maximum_size_bytes = si_mock
        actual = common._get_attribute_value_of(system_obj.simple_storages,
                                                'maximum_size_bytes')
        self.assertEqual(1000169537536, actual)

    def test__get_attribute_value_of_sushy_error(self):
        system_obj = self.system_obj
        si_mock = mock.PropertyMock(side_effect=sushy.exceptions.SushyError)
        type(system_obj.simple_storages).maximum_size_bytes = si_mock
        actual = common._get_attribute_value_of(system_obj.simple_storages,
                                                'maximum_size_bytes',
                                                default=0)
        self.assertEqual(0, actual)

    def test__get_attribute_value_of_fail_missing_attribute(self):
        system_obj = self.system_obj
        si_mock = mock.PropertyMock(
            side_effect=exception.MissingAttributeError)
        type(system_obj.simple_storages).maximum_size_bytes = si_mock
        actual = common._get_attribute_value_of(system_obj.simple_storages,
                                                'maximum_size_bytes')
        self.assertIsNone(actual)

    @ddt.data((True, False, True),
              (True, True, True),
              (False, True, True),
              (False, False, False))
    @ddt.unpack
    def test_has_ssd(self, smart_value, storage_value, expected):
        system_obj = self.system_obj
        type(system_obj.smart_storage).has_ssd = (
            self._mock_property(smart_value))
        type(system_obj.storages).has_ssd = (
            self._mock_property(storage_value))
        actual = common.has_ssd(system_obj)
        self.assertEqual(expected, actual)

    @ddt.data((True, False, True),
              (True, True, True),
              (False, True, True),
              (False, False, False))
    @ddt.unpack
    def test_has_rotational(self, smart_value, storage_value,
                            expected):
        system_obj = self.system_obj
        type(system_obj.smart_storage).has_rotational = (
            self._mock_property(smart_value))
        type(system_obj.storages).has_rotational = (
            self._mock_property(storage_value))
        actual = common.has_rotational(system_obj)
        self.assertEqual(expected, actual)

    @ddt.data((False, False),
              (True, True))
    @ddt.unpack
    def test_has_nvme_ssd(self, storage_value, expected):
        system_obj = self.system_obj
        type(system_obj.storages).has_nvme_ssd = (
            self._mock_property(storage_value))
        actual = common.has_nvme_ssd(system_obj)
        self.assertEqual(expected, actual)

    @ddt.data((set([10000]), set([15000]), set([10000, 15000])),
              (set([10000]), set(), set([10000])),
              (set(), set([15000]), set([15000])),
              (set(), set(), set()))
    @ddt.unpack
    def test_get_drive_rotational_speed_rpm(self, smart_value,
                                            storage_value, expected):
        system_obj = self.system_obj
        type(system_obj.smart_storage).drive_rotational_speed_rpm = (
            self._mock_property(smart_value))
        type(system_obj.storages).drive_rotational_speed_rpm = (
            self._mock_property(storage_value))
        actual = common.get_drive_rotational_speed_rpm(system_obj)
        self.assertEqual(expected, actual)



# Copyright 2017 Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

__author__ = 'HPE'

import json
from unittest import mock

import testtools

from proliantutils.redfish.resources.account_service import account


class HPEAccountCollectionTestCase(testtools.TestCase):

    def setUp(self):
        super(HPEAccountCollectionTestCase, self).setUp()
        self.conn = mock.MagicMock()
        with open('proliantutils/tests/redfish/'
                  'json_samples/account_collection.json', 'r') as f:
            self.conn.get.return_value.json.return_value = (
                json.loads(f.read()))

        self.account_coll_obj = account.HPEAccountCollection(
            self.conn, '/redfish/v1/AccountService/Accounts',
            redfish_version='1.0.2')

    def test_get_member_details(self):
        with open('proliantutils/tests/redfish/'
                  'json_samples/account.json', 'r') as f:
            self.conn.get.return_value.json.return_value = (
                json.loads(f.read()))

        obj = self.account_coll_obj.get_member_details('foo')
        self.assertIsInstance(obj, account.HPEAccount)
        self.assertIsNone(self.account_coll_obj.get_member_details('bar'))


class HPEAccountTestCase(testtools.TestCase):

    def setUp(self):
        super(HPEAccountTestCase, self).setUp()
        self.conn = mock.MagicMock()
        with open('proliantutils/tests/redfish/'
                  'json_samples/account.json', 'r') as f:
            account_json = json.loads(f.read())

        self.conn.get.return_value.json.return_value = account_json

        self.acc_inst = account.HPEAccount(
            self.conn, '/redfish/v1/AccountService/Accounts/1',
            redfish_version='1.0.2')

    def test_attributes(self):
        self.assertEqual('foo', self.acc_inst.username)

    def test_update_credentials(self):
        target_uri = '/redfish/v1/AccountService/Accounts/1'
        self.acc_inst.update_credentials('fake-password')
        self.acc_inst._conn.patch.assert_called_once_with(
            target_uri, data={'Password': 'fake-password'})



# Copyright 2017 Hewlett Packard Enterprise Development LP
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json
from unittest import mock

from sushy import auth as sushy_auth
import testtools

from proliantutils import exception
from proliantutils.redfish import connector
from proliantutils.redfish import main
from proliantutils.redfish.resources.account_service import account_service
from proliantutils.redfish.resources.manager import manager
from proliantutils.redfish.resources.system import system
from proliantutils.redfish.resources import update_service


class HPESushyTestCase(testtools.TestCase):

    @mock.patch.object(sushy_auth, 'SessionOrBasicAuth', autospec=True)
    @mock.patch.object(connector, 'HPEConnector', autospec=True)
    def setUp(self, connector_mock, mock_auth):
        super(HPESushyTestCase, self).setUp()
        with open('proliantutils/tests/redfish/'
                  'json_samples/root.json', 'r') as f:
            root_json = json.loads(f.read())
        connector_mock.return_value.get.return_value.json.return_value = (
            root_json)
        self.hpe_sushy = main.HPESushy('https://1.2.3.4',
                                       username='foo',
                                       password='bar')

    def test_get_system_collection_path(self):
        self.assertEqual('/redfish/v1/Systems/',
                         self.hpe_sushy.get_system_collection_path())

    def test_get_system_collection_path_missing_systems_attr(self):
        self.hpe_sushy.json.pop('Systems')
        self.assertRaisesRegex(
            exception.MissingAttributeError,
            'The attribute Systems is missing',
            self.hpe_sushy.get_system_collection_path)

    @mock.patch.object(system, 'HPESystem', autospec=True)
    def test_get_system(self, mock_system):
        sys_inst = self.hpe_sushy.get_system('1234')
        self.assertIsInstance(sys_inst,
                              system.HPESystem.__class__)
        mock_system.assert_called_once_with(self.hpe_sushy._conn,
                                            '1234',
                                            self.hpe_sushy.redfish_version)

    def test_get_manager_collection_path(self):
        self.assertEqual('/redfish/v1/Managers/',
                         self.hpe_sushy.get_manager_collection_path())

    def test_get_manager_collection_path_missing_systems_attr(self):
        self.hpe_sushy.json.pop('Managers')
        self.assertRaisesRegex(
            exception.MissingAttributeError,
            'The attribute Managers is missing',
            self.hpe_sushy.get_manager_collection_path)

    @mock.patch.object(manager, 'HPEManager', autospec=True)
    def test_get_manager(self, mock_manager):
        sys_inst = self.hpe_sushy.get_manager('1234')
        self.assertIsInstance(sys_inst,
                              manager.HPEManager.__class__)
        mock_manager.assert_called_once_with(self.hpe_sushy._conn,
                                             '1234',
                                             self.hpe_sushy.redfish_version)

    @mock.patch.object(update_service, 'HPEUpdateService', autospec=True)
    def test_get_update_service(self, mock_update_service):
        us_inst = self.hpe_sushy.get_update_service()
        self.assertIsInstance(us_inst,
                              update_service.HPEUpdateService.__class__)
        mock_update_service.assert_called_once_with(
            self.hpe_sushy._conn, "/redfish/v1/UpdateService/",
            self.hpe_sushy.redfish_version)

    @mock.patch.object(account_service, 'HPEAccountService', autospec=True)
    def test_get_account_service(self, mock_account_service):
        acc_inst = self.hpe_sushy.get_account_service()
        self.assertIsInstance(acc_inst,
                              account_service.HPEAccountService.__class__)
        mock_account_service.assert_called_once_with(
            self.hpe_sushy._conn, "/redfish/v1/AccountService/",
            self.hpe_sushy.redfish_version)



# Copyright 2017 Hewlett Packard Enterprise Development LP
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json
from unittest import mock

import testtools

from proliantutils.redfish.resources.system.storage import array_controller
from proliantutils.redfish.resources.system.storage import smart_storage


class HPESmartStorageTestCase(testtools.TestCase):

    def setUp(self):
        super(HPESmartStorageTestCase, self).setUp()
        self.conn = mock.Mock()
        storage_file = ('proliantutils/tests/redfish/json_samples'
                        '/smart_storage.json')
        with open(storage_file, 'r') as f:
            self.json_doc = json.load(f)
        self.conn.get.return_value.json.return_value = self.json_doc

        path = ("/redfish/v1/Systems/1/SmartStorage")
        self.sys_stor = smart_storage.HPESmartStorage(
            self.conn, path, '1.0.2', registries=None, root=None)

    def test__parse_attributes(self):
        self.sys_stor._parse_attributes(self.json_doc)
        self.assertEqual('1.0.2', self.sys_stor.redfish_version)

    def test_array_controllers(self):
        self.conn.get.return_value.json.reset_mock()
        coll = None
        value = None
        path = ('proliantutils/tests/redfish/json_samples/'
                'array_controller_collection.json')
        with open(path, 'r') as f:
            coll = json.loads(f.read())
        with open('proliantutils/tests/redfish/json_samples/'
                  'array_controller.json', 'r') as f:
            value = (json.loads(f.read()))
        self.conn.get.return_value.json.side_effect = [coll, value]
        self.assertIsInstance(self.sys_stor.array_controllers,
                              array_controller.HPEArrayControllerCollection)

    def test_logical_drives_maximum_size_mib(self):
        self.conn.get.return_value.json.reset_mock()
        val = []
        path = ('proliantutils/tests/redfish/json_samples/'
                'array_controller_collection.json')
        with open(path, 'r') as f:
            val.append(json.loads(f.read()))
        path = ('proliantutils/tests/redfish/json_samples/'
                'array_controller.json')
        with open(path, 'r') as f:
            val.append(json.loads(f.read()))
        path = ('proliantutils/tests/redfish/json_samples/'
                'logical_drive_collection.json')
        with open(path, 'r') as f:
            val.append(json.loads(f.read()))
        path = ('proliantutils/tests/redfish/json_samples/'
                'logical_drive.json')
        with open(path, 'r') as f:
            logical_drive = json.loads(f.read())
            val.append(logical_drive['logical_drive1'])
            val.append(logical_drive['logical_drive2'])
            self.conn.get.return_value.json.side_effect = val
        expected = 953837
        actual = self.sys_stor.logical_drives_maximum_size_mib
        self.assertEqual(expected, actual)

    def test_logical_raid_levels(self):
        self.conn.get.return_value.json.reset_mock()
        val = []
        path = ('proliantutils/tests/redfish/json_samples/'
                'array_controller_collection.json')
        with open(path, 'r') as f:
            val.append(json.loads(f.read()))
        path = ('proliantutils/tests/redfish/json_samples/'
                'array_controller.json')
        with open(path, 'r') as f:
            val.append(json.loads(f.read()))
        path = ('proliantutils/tests/redfish/json_samples/'
                'logical_drive_collection.json')
        with open(path, 'r') as f:
            val.append(json.loads(f.read()))
        path = ('proliantutils/tests/redfish/json_samples/'
                'logical_drive.json')
        with open(path, 'r') as f:
            logical_drive = json.loads(f.read())
            val.append(logical_drive['logical_drive1'])
            val.append(logical_drive['logical_drive2'])
            self.conn.get.return_value.json.side_effect = val
        expected = set(['0', '1'])
        actual = self.sys_stor.logical_raid_levels
        self.assertEqual(expected, actual)

    def test_physical_drives_maximum_size_mib(self):
        self.conn.get.return_value.json.reset_mock()
        val = []
        path = ('proliantutils/tests/redfish/json_samples/'
                'array_controller_collection.json')
        with open(path, 'r') as f:
            val.append(json.loads(f.read()))
        path = ('proliantutils/tests/redfish/json_samples/'
                'array_controller.json')
        with open(path, 'r') as f:
            val.append(json.loads(f.read()))
        path = ('proliantutils/tests/redfish/json_samples/'
                'disk_drive_collection.json')
        with open(path, 'r') as f:
            val.append(json.loads(f.read()))
        path = ('proliantutils/tests/redfish/json_samples/'
                'disk_drive.json')
        with open(path, 'r') as f:
            dr_json = json.loads(f.read())
            val.append(dr_json['drive1'])
            val.append(dr_json['drive2'])
            self.conn.get.return_value.json.side_effect = val
        expected = 572325
        actual = self.sys_stor.physical_drives_maximum_size_mib
        self.assertEqual(expected, actual)

    def test_has_ssd(self):
        self.conn.get.return_value.json.reset_mock()
        val = []
        path = ('proliantutils/tests/redfish/json_samples/'
                'array_controller_collection.json')
        with open(path, 'r') as f:
            val.append(json.loads(f.read()))
        path = ('proliantutils/tests/redfish/json_samples/'
                'array_controller.json')
        with open(path, 'r') as f:
            val.append(json.loads(f.read()))
        path = ('proliantutils/tests/redfish/json_samples/'
                'disk_drive_collection.json')
        with open(path, 'r') as f:
            val.append(json.loads(f.read()))
        path = ('proliantutils/tests/redfish/json_samples/'
                'disk_drive.json')
        with open(path, 'r') as f:
            dr_json = json.loads(f.read())
            val.append(dr_json['drive1'])
            val.append(dr_json['drive2'])
            self.conn.get.return_value.json.side_effect = val
        self.assertTrue(self.sys_stor.has_ssd)

    def test_has_rotaional(self):
        self.conn.get.return_value.json.reset_mock()
        val = []
        path = ('proliantutils/tests/redfish/json_samples/'
                'array_controller_collection.json')
        with open(path, 'r') as f:
            val.append(json.loads(f.read()))
        path = ('proliantutils/tests/redfish/json_samples/'
                'array_controller.json')
        with open(path, 'r') as f:
            val.append(json.loads(f.read()))
        path = ('proliantutils/tests/redfish/json_samples/'
                'disk_drive_collection.json')
        with open(path, 'r') as f:
            val.append(json.loads(f.read()))
        path = ('proliantutils/tests/redfish/json_samples/'
                'disk_drive.json')
        with open(path, 'r') as f:
            dr_json = json.loads(f.read())
            val.append(dr_json['drive1'])
            val.append(dr_json['drive2'])
            self.conn.get.return_value.json.side_effect = val
        self.assertTrue(self.sys_stor.has_rotational)

    def test_drive_rotational_speed_rpm(self):
        self.conn.get.return_value.json.reset_mock()
        val = []
        path = ('proliantutils/tests/redfish/json_samples/'
                'array_controller_collection.json')
        with open(path, 'r') as f:
            val.append(json.loads(f.read()))
        path = ('proliantutils/tests/redfish/json_samples/'
                'array_controller.json')
        with open(path, 'r') as f:
            val.append(json.loads(f.read()))
        path = ('proliantutils/tests/redfish/json_samples/'
                'disk_drive_collection.json')
        with open(path, 'r') as f:
            val.append(json.loads(f.read()))
        path = ('proliantutils/tests/redfish/json_samples/'
                'disk_drive.json')
        with open(path, 'r') as f:
            dr_json = json.loads(f.read())
            val.append(dr_json['drive1'])
            val.append(dr_json['drive2'])
            self.conn.get.return_value.json.side_effect = val
        expected = set([10000])
        self.assertEqual(expected,
                         self.sys_stor.drive_rotational_speed_rpm)



# Copyright 2017 Hewlett Packard Enterprise Development LP
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


from sushy.resources import base
from sushy import utils as sushy_utils

from proliantutils.redfish import utils


class SimpleStorage(base.ResourceBase):
    """This class represents the SimpleStorage resource"""

    identity = base.Field('Id', required=True)
    """The SimpleStorage identity string"""

    name = base.Field('Name')
    """The name of the resource or array element"""

    description = base.Field('Description')
    """Description"""

    devices = base.Field('Devices')
    """The storage devices associated with this resource"""

    @property
    @sushy_utils.cache_it
    def maximum_size_bytes(self):
        """Gets the biggest disk drive

        :returns size in bytes.
        """
        return utils.max_safe(
            [device.get('CapacityBytes') for device in self.devices
             if device.get('CapacityBytes') is not None])


class SimpleStorageCollection(base.ResourceCollectionBase):

    @property
    def _resource_type(self):
        return SimpleStorage

    @property
    @sushy_utils.cache_it
    def maximum_size_bytes(self):
        """Gets the biggest disk drive

        :returns size in bytes.
        """
        return utils.max_safe([member.maximum_size_bytes
                               for member in self.get_members()])



# Copyright 2021 Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

__author__ = 'HPE'

import os

import sushy

from proliantutils import exception
from proliantutils import log

LOG = log.get_logger(__name__)


def _get_attribute_value_of(resource, attribute_name, default=None):
    """Gets the value of attribute_name from the resource

    It catches the exception, if any, while retrieving the
    value of attribute_name from resource and returns default.

    :param resource: The resource object
    :attribute_name: Property of the resource
    :returns the property value if no error encountered
        else return 0.
    """
    try:
        return getattr(resource, attribute_name)
    except (sushy.exceptions.SushyError,
            exception.MissingAttributeError) as e:
        msg = (('The Redfish controller failed to get the '
                'attribute %(attribute)s from resource %(resource)s. '
                'Error %(error)s') % {'error': str(e),
                                      'attribute': attribute_name,
                                      'resource':
                                      resource.__class__.__name__})
        LOG.debug(msg)
        return default


def gpu_capabilities(system_obj, chassis_obj):
    """Gets the additional gpu capablities

    The PCIDevice URIs /redfish/v1/Systems/1/PCIDevices/<id>
    has the details for the device if it falls in GPU devices
    classification.
    The Devices URIs /redfish/v1/Chassis/1/Devices/<id>
    has the vendor name, vendor device names and the
    device_instances which has the list of PCIDevice URIs for the
    particular Device ID.
    The logic implemented is:
    1. Get the Chassis Devices URIs data
    2. Check if the PCIDevices given in 'device_instances' corresponds
       to GPU device classification i.e. they are there in the list
       returned by 'gpu_devices'.
    3. If the given Chassis Devices URI is for GPU device,
       then create the capability variables with following:
       gpu_<vendor_name>_count - Numeric
       gpu_<vendor_device_name>_count - Numeric
       gpu_<vendor_device_name> - True/False
    :param system_obj: An instance of sushy system
    :param chassis_obj: An instance of sushy chassis
    :returns list of additional gpu_capabilities.
    """

    gpu_cap = [{'gpu_vendor_count': {}},
               {'gpu_ven_dev_count': {}},
               {'gpu_ven_dev': {}}]

    # Get the list of PCIDevice URIs which are GPU devices
    pci_devices = _get_attribute_value_of(system_obj, 'pci_devices')
    gpu_device_list = _get_attribute_value_of(pci_devices, 'gpu_devices')

    devices = _get_attribute_value_of(chassis_obj, 'devices')

    # Get the dictionary of {<id1>: <vendor_name>, <id2>: <vendor_name>, ...}
    # from Chassis Devices URIs.
    vendor_dict = _get_attribute_value_of(devices, 'vendor_dict')

    # Get the dictionary of
    #        {<id1>: [<pci_uri1>, ...],
    #         <id2>: [<pci_uri2>, ...]}
    # from Chassis Devices URIs.
    pci_devices_uris = _get_attribute_value_of(devices,
                                               'pci_devices_uris')

    # Get the dictionary of {<id1>: <device_name>, ...} from
    # Chassis Devices URIs.
    vendor_devices_dict = _get_attribute_value_of(devices,
                                                  'vendor_devices_dict')

    # Get the dictionary of {<pci_id1>: <vendor_id>, ...} from the
    # PCI Device URIs but only for those devices which are GPU devices.
    vendor_id_dict = _get_attribute_value_of(pci_devices, 'vendor_id')

    for identity in pci_devices_uris:
        devices_list = pci_devices_uris[identity]
        for member in devices_list:
            if member in gpu_device_list:
                vendor_name = vendor_dict[identity]
                count_ven_var = 'gpu_' + vendor_name + '_count'
                # Sometimes there is no vendor name in the redfish output.
                # Hence creating the variable with the numeric vendor
                # ID for those gpu devices
                if vendor_name == 'NULL':
                    pci_uri_id = os.path.basename(member)
                    vendor_name = hex(vendor_id_dict[pci_uri_id])
                    count_ven_var = 'gpu_' + vendor_name + '_count'

                gpu_device_name = vendor_devices_dict[identity]
                # The data returned is like "Embedded Video Controller"
                # Replacing space with underscore to form the correct
                # capability name.
                gpu_dev_name = gpu_device_name.replace(" ", "_")
                count_ven_dev_var = 'gpu_' + gpu_dev_name + '_count'
                ven_dev_var = 'gpu_' + gpu_dev_name

                if count_ven_var not in gpu_cap[0]['gpu_vendor_count']:
                    gpu_cap[0]['gpu_vendor_count'].update({count_ven_var: 1})
                else:
                    gpu_ven_count = (
                        gpu_cap[0]['gpu_vendor_count'].get(count_ven_var))
                    gpu_cap[0]['gpu_vendor_count'].update(
                        {count_ven_var: gpu_ven_count + 1})

                if count_ven_dev_var not in gpu_cap[1]['gpu_ven_dev_count']:
                    gpu_cap[1]['gpu_ven_dev_count'].update(
                        {count_ven_dev_var: 1})
                    gpu_cap[2]['gpu_ven_dev'].update({ven_dev_var: True})
                else:
                    dev_count = (
                        gpu_cap[1]['gpu_ven_dev_count'].get(
                            count_ven_dev_var))
                    gpu_cap[1]['gpu_ven_dev_count'].update(
                        {count_ven_dev_var: dev_count + 1})
    return gpu_cap



# Copyright 2015 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

HPSSA_NO_DRIVES = '''
Smart Array P822 in Slot 2
   Bus Interface: PCI
   Slot: 2
   Serial Number: PDVTF0BRH5T0MO
   Cache Serial Number: PBKUD0BRH5T3I6
   RAID 6 (ADG) Status: Enabled
   Controller Status: OK
   Hardware Revision: B
   Firmware Version: 4.68
   Wait for Cache Room: Disabled
   Surface Analysis Inconsistency Notification: Disabled
   Post Prompt Timeout: 15 secs
   Cache Board Present: True
   Cache Status: OK
   Drive Write Cache: Disabled
   Total Cache Size: 2.0 GB
   Total Cache Memory Available: 1.8 GB
   No-Battery Write Cache: Disabled
   Cache Backup Power Source: Capacitors
   Battery/Capacitor Count: 1
   Battery/Capacitor Status: OK
   SATA NCQ Supported: True
   Spare Activation Mode: Activate on physical drive failure (default)
   Controller Temperature (C): 88
   Cache Module Temperature (C): 37
   Capacitor Temperature  (C): 21
   Number of Ports: 6 (2 Internal / 4 External )
   Driver Name: hpsa
   Driver Version: 3.4.4
   Driver Supports HP SSD Smart Path: True



   unassigned

      physicaldrive 5I:1:1
         Port: 5I
         Box: 1
         Bay: 1
         Status: OK
         Drive Type: Unassigned Drive
         Interface Type: SAS
         Size: 500 GB
         Native Block Size: 512
         Rotational Speed: 15000
         Firmware Revision: HPD6
         Serial Number: 6SL7G55D0000N4173JLT
         Model: HP      EF0600FARNA
         Current Temperature (C): 35
         Maximum Temperature (C): 43
         PHY Count: 2
         PHY Transfer Rate: 6.0Gbps, Unknown
         Drive Authentication Status: OK
         Carrier Application Version: 11
         Carrier Bootloader Version: 6

      physicaldrive 5I:1:2
         Port: 5I
         Box: 1
         Bay: 2
         Status: OK
         Drive Type: Unassigned Drive
         Interface Type: SAS
         Size: 600 GB
         Native Block Size: 512
         Rotational Speed: 15000
         Firmware Revision: HPD6
         Serial Number: 6SL7H2DM0000B41800Y0
         Model: HP      EF0600FARNA
         Current Temperature (C): 35
         Maximum Temperature (C): 44
         PHY Count: 2
         PHY Transfer Rate: 6.0Gbps, Unknown
         Drive Authentication Status: OK
         Carrier Application Version: 11
         Carrier Bootloader Version: 6

      physicaldrive 5I:1:3
         Port: 5I
         Box: 1
         Bay: 3
         Status: OK
         Drive Type: Unassigned Drive
         Interface Type: SAS
         Size: 400 GB
         Native Block Size: 512
         Rotational Speed: 15000
         Firmware Revision: HPD6
         Serial Number: 6SL7G4ZN0000B41707PD
         Model: HP      EF0600FARNA
         Current Temperature (C): 33
         Maximum Temperature (C): 42
         PHY Count: 2
         PHY Transfer Rate: 6.0Gbps, Unknown
         Drive Authentication Status: OK
         Carrier Application Version: 11
         Carrier Bootloader Version: 6

      physicaldrive 5I:1:4
         Port: 5I
         Box: 1
         Bay: 4
         Status: OK
         Drive Type: Unassigned Drive
         Interface Type: SAS
         Size: 400 GB
         Native Block Size: 512
         Rotational Speed: 15000
         Firmware Revision: HPD6
         Serial Number: 6SL7H27F0000B41800S0
         Model: HP      EF0600FARNA
         Current Temperature (C): 36
         Maximum Temperature (C): 45
         PHY Count: 2
         PHY Transfer Rate: 6.0Gbps, Unknown
         Drive Authentication Status: OK
         Carrier Application Version: 11
         Carrier Bootloader Version: 6

      physicaldrive 6I:1:5
         Port: 6I
         Box: 1
         Bay: 5
         Status: OK
         Drive Type: Unassigned Drive
         Interface Type: SAS
         Size: 400 GB
         Native Block Size: 512
         Rotational Speed: 15000
         Firmware Revision: HPD6
         Serial Number: 6SL7H2BR0000B41800V8
         Model: HP      EF0600FARNA
         Current Temperature (C): 32
         Maximum Temperature (C): 41
         PHY Count: 2
         PHY Transfer Rate: 6.0Gbps, Unknown
         Drive Authentication Status: OK
         Carrier Application Version: 11
         Carrier Bootloader Version: 6

      physicaldrive 6I:1:6
         Port: 6I
         Box: 1
         Bay: 6
         Status: OK
         Drive Type: Unassigned Drive
         Interface Type: SAS
         Size: 600 GB
         Native Block Size: 512
         Rotational Speed: 15000
         Firmware Revision: HPD6
         Serial Number: 6SL7G4WD0000N4180GEJ
         Model: HP      EF0600FARNA
         Current Temperature (C): 35
         Maximum Temperature (C): 44
         PHY Count: 2
         PHY Transfer Rate: 6.0Gbps, Unknown
         Drive Authentication Status: OK
         Carrier Application Version: 11
         Carrier Bootloader Version: 6

      physicaldrive 6I:1:7
         Port: 6I
         Box: 1
         Bay: 7
         Status: OK
         Drive Type: Unassigned Drive
         Interface Type: SAS
         Size: 600 GB
         Native Block Size: 512
         Rotational Speed: 15000
         Firmware Revision: HPD6
         Serial Number: 6SL7G54Q0000N4180W34
         Model: HP      EF0600FARNA
         Current Temperature (C): 31
         Maximum Temperature (C): 39
         PHY Count: 2
         PHY Transfer Rate: 6.0Gbps, Unknown
         Drive Authentication Status: OK
         Carrier Application Version: 11
         Carrier Bootloader Version: 6


   SEP (Vendor ID PMCSIERA, Model SRCv24x6G) 380
      Device Number: 380
      Firmware Version: RevB
      WWID: 5001438028842E1F
      Vendor ID: PMCSIERA
      Model: SRCv24x6G
'''

HPSSA_DRIVES_SSD = '''
Smart Array P822 in Slot 2
   Bus Interface: PCI
   Slot: 2
   Serial Number: PDVTF0BRH5T0MO
   Cache Serial Number: PBKUD0BRH5T3I6
   RAID 6 (ADG) Status: Enabled
   Controller Status: OK
   Hardware Revision: B
   Firmware Version: 4.68
   Wait for Cache Room: Disabled
   Surface Analysis Inconsistency Notification: Disabled
   Post Prompt Timeout: 15 secs
   Cache Board Present: True
   Cache Status: OK
   Drive Write Cache: Disabled
   Total Cache Size: 2.0 GB
   Total Cache Memory Available: 1.8 GB
   No-Battery Write Cache: Disabled
   Cache Backup Power Source: Capacitors
   Battery/Capacitor Count: 1
   Battery/Capacitor Status: OK
   SATA NCQ Supported: True
   Spare Activation Mode: Activate on physical drive failure (default)
   Controller Temperature (C): 88
   Cache Module Temperature (C): 37
   Capacitor Temperature  (C): 21
   Number of Ports: 6 (2 Internal / 4 External )
   Driver Name: hpsa
   Driver Version: 3.4.4
   Driver Supports HP SSD Smart Path: True



   unassigned

      physicaldrive 6I:1:7
         Port: 6I
         Box: 1
         Bay: 7
         Status: OK
         Drive Type: Unassigned Drive
         Interface Type: Solid State SAS
         Size: 200 GB
         Native Block Size: 512
         Rotational Speed: 15000
         Firmware Revision: HPD6
         Serial Number: 6SL7G54Q0000N4180W34
         Model: HP      EF0600FARNA
         Current Temperature (C): 31
         Maximum Temperature (C): 39
         PHY Count: 2
         PHY Transfer Rate: 6.0Gbps, Unknown
         Drive Authentication Status: OK
         Carrier Application Version: 11
         Carrier Bootloader Version: 6

      physicaldrive 6I:1:8
         Port: 6I
         Box: 1
         Bay: 7
         Status: OK
         Drive Type: Unassigned Drive
         Interface Type: Solid State SATA
         Size: 200 GB
         Native Block Size: 512
         Rotational Speed: 15000
         Firmware Revision: HPD6
         Serial Number: 6SL7G54Q0000N4180W34
         Model: HP      EF0600FARNA
         Current Temperature (C): 31
         Maximum Temperature (C): 39
         PHY Count: 2
         PHY Transfer Rate: 6.0Gbps, Unknown
         Drive Authentication Status: OK
         Carrier Application Version: 11
         Carrier Bootloader Version: 6


   SEP (Vendor ID PMCSIERA, Model SRCv24x6G) 380
      Device Number: 380
      Firmware Version: RevB
      WWID: 5001438028842E1F
      Vendor ID: PMCSIERA
      Model: SRCv24x6G
'''

HPSSA_ONE_DRIVE = '''
Smart Array P822 in Slot 2
   Bus Interface: PCI
   Slot: 2
   Serial Number: PDVTF0BRH5T0MO
   Cache Serial Number: PBKUD0BRH5T3I6
   RAID 6 (ADG) Status: Enabled
   Controller Status: OK
   Hardware Revision: B
   Firmware Version: 4.68
   Rebuild Priority: Medium
   Expand Priority: Medium
   Surface Scan Delay: 3 secs
   Surface Scan Mode: Idle
   Queue Depth: Automatic
   Monitor and Performance Delay: 60  min
   Elevator Sort: Enabled
   Degraded Performance Optimization: Disabled
   Inconsistency Repair Policy: Disabled
   Wait for Cache Room: Disabled
   Surface Analysis Inconsistency Notification: Disabled
   Post Prompt Timeout: 15 secs
   Cache Board Present: True
   Cache Status: OK
   Cache Ratio: 10% Read / 90% Write
   Drive Write Cache: Disabled
   Total Cache Size: 2.0 GB
   Total Cache Memory Available: 1.8 GB
   No-Battery Write Cache: Disabled
   Cache Backup Power Source: Capacitors
   Battery/Capacitor Count: 1
   Battery/Capacitor Status: OK
   SATA NCQ Supported: True
   Spare Activation Mode: Activate on physical drive failure (default)
   Controller Temperature (C): 88
   Cache Module Temperature (C): 37
   Capacitor Temperature  (C): 22
   Number of Ports: 6 (2 Internal / 4 External )
   Driver Name: hpsa
   Driver Version: 3.4.4
   Driver Supports HP SSD Smart Path: True

   Array: A
      Interface Type: SAS
      Unused Space: 0  MB
      Status: OK
      MultiDomain Status: OK
      Array Type: Data
      HP SSD Smart Path: disable



      Logical Drive: 1
         Size: 558.9 GB
         Fault Tolerance: 1
         Heads: 255
         Sectors Per Track: 32
         Cylinders: 65535
         Strip Size: 256 KB
         Full Stripe Size: 256 KB
         Status: OK
         MultiDomain Status: OK
         Caching:  Enabled
         Unique Identifier: 600508B1001C321CCA06EB7CD847939D
         Disk Name: /dev/sda
         Mount Points: None
         Logical Drive Label: 01F42227PDVTF0BRH5T0MOAB64
         Mirror Group 0:
            physicaldrive 5I:1:1 (port 5I:box 1:bay 1, SAS, 600 GB, OK)
         Mirror Group 1:
            physicaldrive 5I:1:2 (port 5I:box 1:bay 2, SAS, 600 GB, OK)
         Drive Type: Data
         LD Acceleration Method: Controller Cache

      physicaldrive 5I:1:1
         Port: 5I
         Box: 1
         Bay: 1
         Status: OK
         Drive Type: Data Drive
         Interface Type: SAS
         Size: 500 GB
         Native Block Size: 512
         Rotational Speed: 15000
         Firmware Revision: HPD6
         Serial Number: 6SL7G55D0000N4173JLT
         Model: HP      EF0600FARNA
         Current Temperature (C): 35
         Maximum Temperature (C): 43
         PHY Count: 2
         PHY Transfer Rate: 6.0Gbps, Unknown
         Drive Authentication Status: OK
         Carrier Application Version: 11
         Carrier Bootloader Version: 6

      physicaldrive 5I:1:2
         Port: 5I
         Box: 1
         Bay: 2
         Status: OK
         Drive Type: Data Drive
         Interface Type: SAS
         Size: 500 GB
         Native Block Size: 512
         Rotational Speed: 15000
         Firmware Revision: HPD6
         Serial Number: 6SL7H2DM0000B41800Y0
         Model: HP      EF0600FARNA
         Current Temperature (C): 35
         Maximum Temperature (C): 44
         PHY Count: 2
         PHY Transfer Rate: 6.0Gbps, Unknown
         Drive Authentication Status: OK
         Carrier Application Version: 11
         Carrier Bootloader Version: 6


   unassigned

      physicaldrive 5I:1:3
         Port: 5I
         Box: 1
         Bay: 3
         Status: OK
         Drive Type: Unassigned Drive
         Interface Type: SAS
         Size: 400 GB
         Native Block Size: 512
         Rotational Speed: 15000
         Firmware Revision: HPD6
         Serial Number: 6SL7G4ZN0000B41707PD
         Model: HP      EF0600FARNA
         Current Temperature (C): 33
         Maximum Temperature (C): 42
         PHY Count: 2
         PHY Transfer Rate: 6.0Gbps, Unknown
         Drive Authentication Status: OK
         Carrier Application Version: 11
         Carrier Bootloader Version: 6

      physicaldrive 5I:1:4
         Port: 5I
         Box: 1
         Bay: 4
         Status: OK
         Drive Type: Unassigned Drive
         Interface Type: SAS
         Size: 400 GB
         Native Block Size: 512
         Rotational Speed: 15000
         Firmware Revision: HPD6
         Serial Number: 6SL7H27F0000B41800S0
         Model: HP      EF0600FARNA
         Current Temperature (C): 36
         Maximum Temperature (C): 45
         PHY Count: 2
         PHY Transfer Rate: 6.0Gbps, Unknown
         Drive Authentication Status: OK
         Carrier Application Version: 11
         Carrier Bootloader Version: 6

      physicaldrive 6I:1:5
         Port: 6I
         Box: 1
         Bay: 5
         Status: OK
         Drive Type: Unassigned Drive
         Interface Type: SAS
         Size: 400 GB
         Native Block Size: 512
         Rotational Speed: 15000
         Firmware Revision: HPD6
         Serial Number: 6SL7H2BR0000B41800V8
         Model: HP      EF0600FARNA
         Current Temperature (C): 32
         Maximum Temperature (C): 41
         PHY Count: 2
         PHY Transfer Rate: 6.0Gbps, Unknown
         Drive Authentication Status: OK
         Carrier Application Version: 11
         Carrier Bootloader Version: 6

      physicaldrive 6I:1:6
         Port: 6I
         Box: 1
         Bay: 6
         Status: OK
         Drive Type: Unassigned Drive
         Interface Type: SAS
         Size: 600 GB
         Native Block Size: 512
         Rotational Speed: 15000
         Firmware Revision: HPD6
         Serial Number: 6SL7G4WD0000N4180GEJ
         Model: HP      EF0600FARNA
         Current Temperature (C): 35
         Maximum Temperature (C): 44
         PHY Count: 2
         PHY Transfer Rate: 6.0Gbps, Unknown
         Drive Authentication Status: OK
         Carrier Application Version: 11
         Carrier Bootloader Version: 6

      physicaldrive 6I:1:7
         Port: 6I
         Box: 1
         Bay: 7
         Status: OK
         Drive Type: Unassigned Drive
         Interface Type: SAS
         Size: 600 GB
         Native Block Size: 512
         Rotational Speed: 15000
         Firmware Revision: HPD6
         Serial Number: 6SL7G54Q0000N4180W34
         Model: HP      EF0600FARNA
         Current Temperature (C): 31
         Maximum Temperature (C): 39
         PHY Count: 2
         PHY Transfer Rate: 6.0Gbps, Unknown
         Drive Authentication Status: OK
         Carrier Application Version: 11
         Carrier Bootloader Version: 6


   SEP (Vendor ID PMCSIERA, Model SRCv24x6G) 380
      Device Number: 380
      Firmware Version: RevB
      WWID: 5001438028842E1F
      Vendor ID: PMCSIERA
      Model: SRCv24x6G
'''


HPSSA_ONE_DRIVE_RAID_50 = '''

Smart Array P822 in Slot 2
   Bus Interface: PCI
   Slot: 2
   Serial Number: PDVTF0BRH5T0MO
   Cache Serial Number: PBKUD0BRH5T3I6
   RAID 6 (ADG) Status: Enabled
   Controller Status: OK
   Hardware Revision: B
   Firmware Version: 4.68
   Rebuild Priority: Medium
   Expand Priority: Medium
   Surface Scan Delay: 3 secs
   Surface Scan Mode: Idle
   Queue Depth: Automatic
   Monitor and Performance Delay: 60  min
   Elevator Sort: Enabled
   Degraded Performance Optimization: Disabled
   Inconsistency Repair Policy: Disabled
   Wait for Cache Room: Disabled
   Surface Analysis Inconsistency Notification: Disabled
   Post Prompt Timeout: 15 secs
   Cache Board Present: True
   Cache Status: OK
   Cache Ratio: 10% Read / 90% Write
   Drive Write Cache: Disabled
   Total Cache Size: 2.0 GB
   Total Cache Memory Available: 1.8 GB
   No-Battery Write Cache: Disabled
   Cache Backup Power Source: Capacitors
   Battery/Capacitor Count: 1
   Battery/Capacitor Status: OK
   SATA NCQ Supported: True
   Spare Activation Mode: Activate on physical drive failure (default)
   Controller Temperature (C): 88
   Cache Module Temperature (C): 38
   Capacitor Temperature  (C): 23
   Number of Ports: 6 (2 Internal / 4 External )
   Driver Name: hpsa
   Driver Version: 3.4.4
   Driver Supports HP SSD Smart Path: True

   Array: A
      Interface Type: SAS
      Unused Space: 3280165  MB
      Status: OK
      MultiDomain Status: OK
      Array Type: Data
      HP SSD Smart Path: disable



      Logical Drive: 1
         Size: 100.0 GB
         Fault Tolerance: 50
         Number of Parity Groups: 2
         Heads: 255
         Sectors Per Track: 32
         Cylinders: 25700
         Strip Size: 256 KB
         Full Stripe Size: 512 KB
         Status: OK
         MultiDomain Status: OK
         Caching:  Enabled
         Parity Initialization Status: Queued
         Unique Identifier: 600508B1001C0FC2145AA6F3A0AF2A57
         Disk Name: /dev/sda
         Mount Points: None
         Logical Drive Label: 02795E8FPDVTF0BRH5T0MOF6B8
         Parity Group 0:
            physicaldrive 5I:1:1 (port 5I:box 1:bay 1, SAS, 600 GB, OK)
            physicaldrive 5I:1:3 (port 5I:box 1:bay 3, SAS, 600 GB, OK)
            physicaldrive 6I:1:5 (port 6I:box 1:bay 5, SAS, 600 GB, OK)
         Parity Group 1:
            physicaldrive 5I:1:2 (port 5I:box 1:bay 2, SAS, 600 GB, OK)
            physicaldrive 5I:1:4 (port 5I:box 1:bay 4, SAS, 600 GB, OK)
            physicaldrive 6I:1:6 (port 6I:box 1:bay 6, SAS, 600 GB, OK)
         Drive Type: Data
         LD Acceleration Method: Controller Cache

      physicaldrive 5I:1:1
         Port: 5I
         Box: 1
         Bay: 1
         Status: OK
         Drive Type: Data Drive
         Interface Type: SAS
         Size: 600 GB
         Native Block Size: 512
         Rotational Speed: 15000
         Firmware Revision: HPD6
         Serial Number: 6SL7G55D0000N4173JLT
         Model: HP      EF0600FARNA
         Current Temperature (C): 36
         Maximum Temperature (C): 43
         PHY Count: 2
         PHY Transfer Rate: 6.0Gbps, Unknown
         Drive Authentication Status: OK
         Carrier Application Version: 11
         Carrier Bootloader Version: 6

      physicaldrive 5I:1:2
         Port: 5I
         Box: 1
         Bay: 2
         Status: OK
         Drive Type: Data Drive
         Interface Type: SAS
         Size: 600 GB
         Native Block Size: 512
         Rotational Speed: 15000
         Firmware Revision: HPD6
         Serial Number: 6SL7H2DM0000B41800Y0
         Model: HP      EF0600FARNA
         Current Temperature (C): 37
         Maximum Temperature (C): 44
         PHY Count: 2
         PHY Transfer Rate: 6.0Gbps, Unknown
         Drive Authentication Status: OK
         Carrier Application Version: 11
         Carrier Bootloader Version: 6

      physicaldrive 5I:1:3
         Port: 5I
         Box: 1
         Bay: 3
         Status: OK
         Drive Type: Data Drive
         Interface Type: SAS
         Size: 600 GB
         Native Block Size: 512
         Rotational Speed: 15000
         Firmware Revision: HPD6
         Serial Number: 6SL7G4ZN0000B41707PD
         Model: HP      EF0600FARNA
         Current Temperature (C): 35
         Maximum Temperature (C): 42
         PHY Count: 2
         PHY Transfer Rate: 6.0Gbps, Unknown
         Drive Authentication Status: OK
         Carrier Application Version: 11
         Carrier Bootloader Version: 6

      physicaldrive 5I:1:4
         Port: 5I
         Box: 1
         Bay: 4
         Status: OK
         Drive Type: Data Drive
         Interface Type: SAS
         Size: 600 GB
         Native Block Size: 512
         Rotational Speed: 15000
         Firmware Revision: HPD6
         Serial Number: 6SL7H27F0000B41800S0
         Model: HP      EF0600FARNA
         Current Temperature (C): 38
         Maximum Temperature (C): 45
         PHY Count: 2
         PHY Transfer Rate: 6.0Gbps, Unknown
         Drive Authentication Status: OK
         Carrier Application Version: 11
         Carrier Bootloader Version: 6

      physicaldrive 6I:1:5
         Port: 6I
         Box: 1
         Bay: 5
         Status: OK
         Drive Type: Data Drive
         Interface Type: SAS
         Size: 600 GB
         Native Block Size: 512
         Rotational Speed: 15000
         Firmware Revision: HPD6
         Serial Number: 6SL7H2BR0000B41800V8
         Model: HP      EF0600FARNA
         Current Temperature (C): 33
         Maximum Temperature (C): 41
         PHY Count: 2
         PHY Transfer Rate: 6.0Gbps, Unknown
         Drive Authentication Status: OK
         Carrier Application Version: 11
         Carrier Bootloader Version: 6

      physicaldrive 6I:1:6
         Port: 6I
         Box: 1
         Bay: 6
         Status: OK
         Drive Type: Data Drive
         Interface Type: SAS
         Size: 600 GB
         Native Block Size: 512
         Rotational Speed: 15000
         Firmware Revision: HPD6
         Serial Number: 6SL7G4WD0000N4180GEJ
         Model: HP      EF0600FARNA
         Current Temperature (C): 36
         Maximum Temperature (C): 44
         PHY Count: 2
         PHY Transfer Rate: 6.0Gbps, Unknown
         Drive Authentication Status: OK
         Carrier Application Version: 11
         Carrier Bootloader Version: 6


   unassigned

      physicaldrive 6I:1:7
         Port: 6I
         Box: 1
         Bay: 7
         Status: OK
         Drive Type: Unassigned Drive
         Interface Type: SAS
         Size: 600 GB
         Native Block Size: 512
         Rotational Speed: 15000
         Firmware Revision: HPD6
         Serial Number: 6SL7G54Q0000N4180W34
         Model: HP      EF0600FARNA
         Current Temperature (C): 33
         Maximum Temperature (C): 39
         PHY Count: 2
         PHY Transfer Rate: 6.0Gbps, Unknown
         Drive Authentication Status: OK
         Carrier Application Version: 11
         Carrier Bootloader Version: 6


   SEP (Vendor ID PMCSIERA, Model SRCv24x6G) 380
      Device Number: 380
      Firmware Version: RevB
      WWID: 5001438028842E1F
      Vendor ID: PMCSIERA
      Model: SRCv24x6G

'''

HPSSA_ONE_DRIVE_100GB_RAID_5 = '''

Smart Array P822 in Slot 2
   Bus Interface: PCI
   Slot: 2
   Serial Number: PDVTF0BRH5T0MO
   Cache Serial Number: PBKUD0BRH5T3I6
   RAID 6 (ADG) Status: Enabled
   Controller Status: OK
   Hardware Revision: B
   Firmware Version: 4.68
   Rebuild Priority: Medium
   Expand Priority: Medium
   Surface Scan Delay: 3 secs
   Surface Scan Mode: Idle
   Queue Depth: Automatic
   Monitor and Performance Delay: 60  min
   Elevator Sort: Enabled
   Degraded Performance Optimization: Disabled
   Inconsistency Repair Policy: Disabled
   Wait for Cache Room: Disabled
   Surface Analysis Inconsistency Notification: Disabled
   Post Prompt Timeout: 15 secs
   Cache Board Present: True
   Cache Status: OK
   Cache Ratio: 10% Read / 90% Write
   Drive Write Cache: Disabled
   Total Cache Size: 2.0 GB
   Total Cache Memory Available: 1.8 GB
   No-Battery Write Cache: Disabled
   Cache Backup Power Source: Capacitors
   Battery/Capacitor Count: 1
   Battery/Capacitor Status: OK
   SATA NCQ Supported: True
   Spare Activation Mode: Activate on physical drive failure (default)
   Controller Temperature (C): 88
   Cache Module Temperature (C): 38
   Capacitor Temperature  (C): 23
   Number of Ports: 6 (2 Internal / 4 External )
   Driver Name: hpsa
   Driver Version: 3.4.4
   Driver Supports HP SSD Smart Path: True

   Array: A
      Interface Type: SAS
      Unused Space: 1563284  MB
      Status: OK
      MultiDomain Status: OK
      Array Type: Data
      HP SSD Smart Path: disable



      Logical Drive: 1
         Size: 100.0 GB
         Fault Tolerance: 5
         Heads: 255
         Sectors Per Track: 32
         Cylinders: 25700
         Strip Size: 256 KB
         Full Stripe Size: 512 KB
         Status: OK
         MultiDomain Status: OK
         Caching:  Enabled
         Parity Initialization Status: Queued
         Unique Identifier: 600508B1001CC42CDF101F06E5563967
         Disk Name: /dev/sda
         Mount Points: None
         Logical Drive Label: 02715627PDVTF0BRH5T0MO154D
         Drive Type: Data
         LD Acceleration Method: Controller Cache

      physicaldrive 5I:1:3
         Port: 5I
         Box: 1
         Bay: 3
         Status: OK
         Drive Type: Data Drive
         Interface Type: SAS
         Size: 400 GB
         Native Block Size: 512
         Rotational Speed: 15000
         Firmware Revision: HPD6
         Serial Number: 6SL7G4ZN0000B41707PD
         Model: HP      EF0600FARNA
         Current Temperature (C): 35
         Maximum Temperature (C): 42
         PHY Count: 2
         PHY Transfer Rate: 6.0Gbps, Unknown
         Drive Authentication Status: OK
         Carrier Application Version: 11
         Carrier Bootloader Version: 6

      physicaldrive 5I:1:4
         Port: 5I
         Box: 1
         Bay: 4
         Status: OK
         Drive Type: Data Drive
         Interface Type: SAS
         Size: 400 GB
         Native Block Size: 512
         Rotational Speed: 15000
         Firmware Revision: HPD6
         Serial Number: 6SL7H27F0000B41800S0
         Model: HP      EF0600FARNA
         Current Temperature (C): 38
         Maximum Temperature (C): 45
         PHY Count: 2
         PHY Transfer Rate: 6.0Gbps, Unknown
         Drive Authentication Status: OK
         Carrier Application Version: 11
         Carrier Bootloader Version: 6

      physicaldrive 6I:1:5
         Port: 6I
         Box: 1
         Bay: 5
         Status: OK
         Drive Type: Data Drive
         Interface Type: SAS
         Size: 400 GB
         Native Block Size: 512
         Rotational Speed: 15000
         Firmware Revision: HPD6
         Serial Number: 6SL7H2BR0000B41800V8
         Model: HP      EF0600FARNA
         Current Temperature (C): 34
         Maximum Temperature (C): 41
         PHY Count: 2
         PHY Transfer Rate: 6.0Gbps, Unknown
         Drive Authentication Status: OK
         Carrier Application Version: 11
         Carrier Bootloader Version: 6


   unassigned

      physicaldrive 5I:1:1
         Port: 5I
         Box: 1
         Bay: 1
         Status: OK
         Drive Type: Unassigned Drive
         Interface Type: SAS
         Size: 500 GB
         Native Block Size: 512
         Rotational Speed: 15000
         Firmware Revision: HPD6
         Serial Number: 6SL7G55D0000N4173JLT
         Model: HP      EF0600FARNA
         Current Temperature (C): 36
         Maximum Temperature (C): 43
         PHY Count: 2
         PHY Transfer Rate: 6.0Gbps, Unknown
         Drive Authentication Status: OK
         Carrier Application Version: 11
         Carrier Bootloader Version: 6

      physicaldrive 5I:1:2
         Port: 5I
         Box: 1
         Bay: 2
         Status: OK
         Drive Type: Unassigned Drive
         Interface Type: SAS
         Size: 500 GB
         Native Block Size: 512
         Rotational Speed: 15000
         Firmware Revision: HPD6
         Serial Number: 6SL7H2DM0000B41800Y0
         Model: HP      EF0600FARNA
         Current Temperature (C): 37
         Maximum Temperature (C): 44
         PHY Count: 2
         PHY Transfer Rate: 6.0Gbps, Unknown
         Drive Authentication Status: OK
         Carrier Application Version: 11
         Carrier Bootloader Version: 6

      physicaldrive 6I:1:6
         Port: 6I
         Box: 1
         Bay: 6
         Status: OK
         Drive Type: Unassigned Drive
         Interface Type: SAS
         Size: 600 GB
         Native Block Size: 512
         Rotational Speed: 15000
         Firmware Revision: HPD6
         Serial Number: 6SL7G4WD0000N4180GEJ
         Model: HP      EF0600FARNA
         Current Temperature (C): 36
         Maximum Temperature (C): 44
         PHY Count: 2
         PHY Transfer Rate: 6.0Gbps, Unknown
         Drive Authentication Status: OK
         Carrier Application Version: 11
         Carrier Bootloader Version: 6

      physicaldrive 6I:1:7
         Port: 6I
         Box: 1
         Bay: 7
         Status: OK
         Drive Type: Unassigned Drive
         Interface Type: SAS
         Size: 600 GB
         Native Block Size: 512
         Rotational Speed: 15000
         Firmware Revision: HPD6
         Serial Number: 6SL7G54Q0000N4180W34
         Model: HP      EF0600FARNA
         Current Temperature (C): 33
         Maximum Temperature (C): 39
         PHY Count: 2
         PHY Transfer Rate: 6.0Gbps, Unknown
         Drive Authentication Status: OK
         Carrier Application Version: 11
         Carrier Bootloader Version: 6


   SEP (Vendor ID PMCSIERA, Model SRCv24x6G) 380
      Device Number: 380
      Firmware Version: RevB
      WWID: 5001438028842E1F
      Vendor ID: PMCSIERA
      Model: SRCv24x6G
'''


HPSSA_TWO_DRIVES_100GB_RAID5_50GB_RAID1 = '''

Smart Array P822 in Slot 2
   Bus Interface: PCI
   Slot: 2
   Serial Number: PDVTF0BRH5T0MO
   Cache Serial Number: PBKUD0BRH5T3I6
   RAID 6 (ADG) Status: Enabled
   Controller Status: OK
   Hardware Revision: B
   Firmware Version: 4.68
   Rebuild Priority: Medium
   Expand Priority: Medium
   Surface Scan Delay: 3 secs
   Surface Scan Mode: Idle
   Queue Depth: Automatic
   Monitor and Performance Delay: 60  min
   Elevator Sort: Enabled
   Degraded Performance Optimization: Disabled
   Inconsistency Repair Policy: Disabled
   Wait for Cache Room: Disabled
   Surface Analysis Inconsistency Notification: Disabled
   Post Prompt Timeout: 15 secs
   Cache Board Present: True
   Cache Status: OK
   Cache Ratio: 10% Read / 90% Write
   Drive Write Cache: Disabled
   Total Cache Size: 2.0 GB
   Total Cache Memory Available: 1.8 GB
   No-Battery Write Cache: Disabled
   Cache Backup Power Source: Capacitors
   Battery/Capacitor Count: 1
   Battery/Capacitor Status: OK
   SATA NCQ Supported: True
   Spare Activation Mode: Activate on physical drive failure (default)
   Controller Temperature (C): 88
   Cache Module Temperature (C): 38
   Capacitor Temperature  (C): 23
   Number of Ports: 6 (2 Internal / 4 External )
   Driver Name: hpsa
   Driver Version: 3.4.4
   Driver Supports HP SSD Smart Path: True

   Array: A
      Interface Type: SAS
      Unused Space: 1563284  MB
      Status: OK
      MultiDomain Status: OK
      Array Type: Data
      HP SSD Smart Path: disable



      Logical Drive: 1
         Size: 100.0 GB
         Fault Tolerance: 5
         Heads: 255
         Sectors Per Track: 32
         Cylinders: 25700
         Strip Size: 256 KB
         Full Stripe Size: 512 KB
         Status: OK
         MultiDomain Status: OK
         Caching:  Enabled
         Parity Initialization Status: Queued
         Unique Identifier: 600508B1001CC42CDF101F06E5563967
         Disk Name: /dev/sda
         Mount Points: None
         Logical Drive Label: 02715627PDVTF0BRH5T0MO154D
         Drive Type: Data
         LD Acceleration Method: Controller Cache

      physicaldrive 5I:1:3
         Port: 5I
         Box: 1
         Bay: 3
         Status: OK
         Drive Type: Data Drive
         Interface Type: SAS
         Size: 400 GB
         Native Block Size: 512
         Rotational Speed: 15000
         Firmware Revision: HPD6
         Serial Number: 6SL7G4ZN0000B41707PD
         Model: HP      EF0600FARNA
         Current Temperature (C): 35
         Maximum Temperature (C): 42
         PHY Count: 2
         PHY Transfer Rate: 6.0Gbps, Unknown
         Drive Authentication Status: OK
         Carrier Application Version: 11
         Carrier Bootloader Version: 6

      physicaldrive 5I:1:4
         Port: 5I
         Box: 1
         Bay: 4
         Status: OK
         Drive Type: Data Drive
         Interface Type: SAS
         Size: 400 GB
         Native Block Size: 512
         Rotational Speed: 15000
         Firmware Revision: HPD6
         Serial Number: 6SL7H27F0000B41800S0
         Model: HP      EF0600FARNA
         Current Temperature (C): 38
         Maximum Temperature (C): 45
         PHY Count: 2
         PHY Transfer Rate: 6.0Gbps, Unknown
         Drive Authentication Status: OK
         Carrier Application Version: 11
         Carrier Bootloader Version: 6

      physicaldrive 6I:1:5
         Port: 6I
         Box: 1
         Bay: 5
         Status: OK
         Drive Type: Data Drive
         Interface Type: SAS
         Size: 400 GB
         Native Block Size: 512
         Rotational Speed: 15000
         Firmware Revision: HPD6
         Serial Number: 6SL7H2BR0000B41800V8
         Model: HP      EF0600FARNA
         Current Temperature (C): 34
         Maximum Temperature (C): 41
         PHY Count: 2
         PHY Transfer Rate: 6.0Gbps, Unknown
         Drive Authentication Status: OK
         Carrier Application Version: 11
         Carrier Bootloader Version: 6


   Array: B
      Interface Type: SAS
      Unused Space: 1042189  MB
      Status: OK
      MultiDomain Status: OK
      Array Type: Data
      HP SSD Smart Path: disable



      Logical Drive: 2
         Size: 50.0 GB
         Fault Tolerance: 1
         Heads: 255
         Sectors Per Track: 32
         Cylinders: 12850
         Strip Size: 256 KB
         Full Stripe Size: 256 KB
         Status: OK
         MultiDomain Status: OK
         Caching:  Enabled
         Unique Identifier: 600508B1001CE1E18302A8702C6AB008
         Disk Name: /dev/sdb
         Mount Points: None
         Logical Drive Label: 06715654PDVTF0BRH5T0MOACF0
         Mirror Group 0:
            physicaldrive 5I:1:1 (port 5I:box 1:bay 1, SAS, 600 GB, OK)
         Mirror Group 1:
            physicaldrive 5I:1:2 (port 5I:box 1:bay 2, SAS, 600 GB, OK)
         Drive Type: Data
         LD Acceleration Method: Controller Cache

      physicaldrive 5I:1:1
         Port: 5I
         Box: 1
         Bay: 1
         Status: OK
         Drive Type: Data Drive
         Interface Type: SAS
         Size: 500 GB
         Native Block Size: 512
         Rotational Speed: 15000
         Firmware Revision: HPD6
         Serial Number: 6SL7G55D0000N4173JLT
         Model: HP      EF0600FARNA
         Current Temperature (C): 36
         Maximum Temperature (C): 43
         PHY Count: 2
         PHY Transfer Rate: 6.0Gbps, Unknown
         Drive Authentication Status: OK
         Carrier Application Version: 11
         Carrier Bootloader Version: 6

      physicaldrive 5I:1:2
         Port: 5I
         Box: 1
         Bay: 2
         Status: OK
         Drive Type: Data Drive
         Interface Type: SAS
         Size: 500 GB
         Native Block Size: 512
         Rotational Speed: 15000
         Firmware Revision: HPD6
         Serial Number: 6SL7H2DM0000B41800Y0
         Model: HP      EF0600FARNA
         Current Temperature (C): 37
         Maximum Temperature (C): 44
         PHY Count: 2
         PHY Transfer Rate: 6.0Gbps, Unknown
         Drive Authentication Status: OK
         Carrier Application Version: 11
         Carrier Bootloader Version: 6


   unassigned

      physicaldrive 6I:1:6
         Port: 6I
         Box: 1
         Bay: 6
         Status: OK
         Drive Type: Unassigned Drive
         Interface Type: SAS
         Size: 600 GB
         Native Block Size: 512
         Rotational Speed: 15000
         Firmware Revision: HPD6
         Serial Number: 6SL7G4WD0000N4180GEJ
         Model: HP      EF0600FARNA
         Current Temperature (C): 36
         Maximum Temperature (C): 44
         PHY Count: 2
         PHY Transfer Rate: 6.0Gbps, Unknown
         Drive Authentication Status: OK
         Carrier Application Version: 11
         Carrier Bootloader Version: 6

      physicaldrive 6I:1:7
         Port: 6I
         Box: 1
         Bay: 7
         Status: OK
         Drive Type: Unassigned Drive
         Interface Type: SAS
         Size: 600 GB
         Native Block Size: 512
         Rotational Speed: 15000
         Firmware Revision: HPD6
         Serial Number: 6SL7G54Q0000N4180W34
         Model: HP      EF0600FARNA
         Current Temperature (C): 33
         Maximum Temperature (C): 39
         PHY Count: 2
         PHY Transfer Rate: 6.0Gbps, Unknown
         Drive Authentication Status: OK
         Carrier Application Version: 11
         Carrier Bootloader Version: 6


   SEP (Vendor ID PMCSIERA, Model SRCv24x6G) 380
      Device Number: 380
      Firmware Version: RevB
      WWID: 5001438028842E1F
      Vendor ID: PMCSIERA
      Model: SRCv24x6G
'''

HPSSA_BAD_SIZE_PHYSICAL_DRIVE = '''

Smart Array P822 in Slot 2
   Bus Interface: PCI
   Slot: 2
   Serial Number: PDVTF0BRH5T0MO
   Cache Serial Number: PBKUD0BRH5T3I6
   RAID 6 (ADG) Status: Enabled
   Controller Status: OK
   Hardware Revision: B
   Firmware Version: 4.68
   Wait for Cache Room: Disabled
   Surface Analysis Inconsistency Notification: Disabled
   Post Prompt Timeout: 15 secs
   Cache Board Present: True
   Cache Status: OK
   Drive Write Cache: Disabled
   Total Cache Size: 2.0 GB
   Total Cache Memory Available: 1.8 GB
   No-Battery Write Cache: Disabled
   Cache Backup Power Source: Capacitors
   Battery/Capacitor Count: 1
   Battery/Capacitor Status: OK
   SATA NCQ Supported: True
   Spare Activation Mode: Activate on physical drive failure (default)
   Controller Temperature (C): 88
   Cache Module Temperature (C): 37
   Capacitor Temperature  (C): 21
   Number of Ports: 6 (2 Internal / 4 External )
   Driver Name: hpsa
   Driver Version: 3.4.4
   Driver Supports HP SSD Smart Path: True



   unassigned

      physicaldrive 5I:1:1
         Port: 5I
         Box: 1
         Bay: 1
         Status: OK
         Drive Type: Unassigned Drive
         Interface Type: SAS
         Size: 500foo
         Native Block Size: 512
         Rotational Speed: 15000
         Firmware Revision: HPD6
         Serial Number: 6SL7G55D0000N4173JLT
         Model: HP      EF0600FARNA
         Current Temperature (C): 35
         Maximum Temperature (C): 43
         PHY Count: 2
         PHY Transfer Rate: 6.0Gbps, Unknown
         Drive Authentication Status: OK
         Carrier Application Version: 11
         Carrier Bootloader Version: 6
'''


HPSSA_BAD_SIZE_LOGICAL_DRIVE = '''

Smart Array P822 in Slot 2
   Bus Interface: PCI
   Slot: 2
   Serial Number: PDVTF0BRH5T0MO
   Cache Serial Number: PBKUD0BRH5T3I6
   RAID 6 (ADG) Status: Enabled
   Controller Status: OK
   Hardware Revision: B
   Firmware Version: 4.68
   Rebuild Priority: Medium
   Expand Priority: Medium
   Surface Scan Delay: 3 secs
   Surface Scan Mode: Idle
   Queue Depth: Automatic
   Monitor and Performance Delay: 60  min
   Elevator Sort: Enabled
   Degraded Performance Optimization: Disabled
   Inconsistency Repair Policy: Disabled
   Wait for Cache Room: Disabled
   Surface Analysis Inconsistency Notification: Disabled
   Post Prompt Timeout: 15 secs
   Cache Board Present: True
   Cache Status: OK
   Cache Ratio: 10% Read / 90% Write
   Drive Write Cache: Disabled
   Total Cache Size: 2.0 GB
   Total Cache Memory Available: 1.8 GB
   No-Battery Write Cache: Disabled
   Cache Backup Power Source: Capacitors
   Battery/Capacitor Count: 1
   Battery/Capacitor Status: OK
   SATA NCQ Supported: True
   Spare Activation Mode: Activate on physical drive failure (default)
   Controller Temperature (C): 88
   Cache Module Temperature (C): 37
   Capacitor Temperature  (C): 22
   Number of Ports: 6 (2 Internal / 4 External )
   Driver Name: hpsa
   Driver Version: 3.4.4
   Driver Supports HP SSD Smart Path: True

   Array: A
      Interface Type: SAS
      Unused Space: 0  MB
      Status: OK
      MultiDomain Status: OK
      Array Type: Data
      HP SSD Smart Path: disable



      Logical Drive: 1
         Size: 558.9foo
         Fault Tolerance: 1
         Heads: 255
         Sectors Per Track: 32
         Cylinders: 65535
         Strip Size: 256 KB
         Full Stripe Size: 256 KB
         Status: OK
         MultiDomain Status: OK
         Caching:  Enabled
         Unique Identifier: 600508B1001C321CCA06EB7CD847939D
         Disk Name: /dev/sda
         Mount Points: None
         Logical Drive Label: 01F42227PDVTF0BRH5T0MOAB64
         Mirror Group 0:
            physicaldrive 5I:1:1 (port 5I:box 1:bay 1, SAS, 600 GB, OK)
         Mirror Group 1:
            physicaldrive 5I:1:2 (port 5I:box 1:bay 2, SAS, 600 GB, OK)
         Drive Type: Data
         LD Acceleration Method: Controller Cache

      physicaldrive 5I:1:1
         Port: 5I
         Box: 1
         Bay: 1
         Status: OK
         Drive Type: Data Drive
         Interface Type: SAS
         Size: 500 GB
         Native Block Size: 512
         Rotational Speed: 15000
         Firmware Revision: HPD6
         Serial Number: 6SL7G55D0000N4173JLT
         Model: HP      EF0600FARNA
         Current Temperature (C): 35
         Maximum Temperature (C): 43
         PHY Count: 2
         PHY Transfer Rate: 6.0Gbps, Unknown
         Drive Authentication Status: OK
         Carrier Application Version: 11
         Carrier Bootloader Version: 6

      physicaldrive 5I:1:2
         Port: 5I
         Box: 1
         Bay: 2
         Status: OK
         Drive Type: Data Drive
         Interface Type: SAS
         Size: 500 GB
         Native Block Size: 512
         Rotational Speed: 15000
         Firmware Revision: HPD6
         Serial Number: 6SL7H2DM0000B41800Y0
         Model: HP      EF0600FARNA
         Current Temperature (C): 35
         Maximum Temperature (C): 44
         PHY Count: 2
         PHY Transfer Rate: 6.0Gbps, Unknown
         Drive Authentication Status: OK
         Carrier Application Version: 11
         Carrier Bootloader Version: 6
'''

HPSSA_SMALL_SIZE_PHYSICAL_DRIVE = '''

Smart Array P822 in Slot 2
   Bus Interface: PCI
   Slot: 2
   Serial Number: PDVTF0BRH5T0MO
   Cache Serial Number: PBKUD0BRH5T3I6
   RAID 6 (ADG) Status: Enabled
   Controller Status: OK
   Hardware Revision: B
   Firmware Version: 4.68
   Wait for Cache Room: Disabled
   Surface Analysis Inconsistency Notification: Disabled
   Post Prompt Timeout: 15 secs
   Cache Board Present: True
   Cache Status: OK
   Drive Write Cache: Disabled
   Total Cache Size: 2.0 GB
   Total Cache Memory Available: 1.8 GB
   No-Battery Write Cache: Disabled
   Cache Backup Power Source: Capacitors
   Battery/Capacitor Count: 1
   Battery/Capacitor Status: OK
   SATA NCQ Supported: True
   Spare Activation Mode: Activate on physical drive failure (default)
   Controller Temperature (C): 88
   Cache Module Temperature (C): 37
   Capacitor Temperature  (C): 21
   Number of Ports: 6 (2 Internal / 4 External )
   Driver Name: hpsa
   Driver Version: 3.4.4
   Driver Supports HP SSD Smart Path: True



   unassigned

      physicaldrive 5I:1:1
         Port: 5I
         Box: 1
         Bay: 1
         Status: OK
         Drive Type: Unassigned Drive
         Interface Type: SAS
         Size: 2048 MB
         Native Block Size: 512
         Rotational Speed: 15000
         Firmware Revision: HPD6
         Serial Number: 6SL7G55D0000N4173JLT
         Model: HP      EF0600FARNA
         Current Temperature (C): 35
         Maximum Temperature (C): 43
         PHY Count: 2
         PHY Transfer Rate: 6.0Gbps, Unknown
         Drive Authentication Status: OK
         Carrier Application Version: 11
         Carrier Bootloader Version: 6
'''

ARRAY_ACCOMODATE_LOGICAL_DISK = '''

Available options are:
   Max: 1042188 (Units in MB)
   Min: 16 (Units in MB)

'''

ARRAY_ACCOMODATE_LOGICAL_DISK_INVALID = '''

Error: "raid=1" is not a valid option for array A

Available options are:
       0
       1adm
       5 (default value)

'''

HPSSA_NO_DRIVES_3_PHYSICAL_DISKS = '''
Smart Array P822 in Slot 2
   Bus Interface: PCI
   Slot: 2
   Serial Number: PDVTF0BRH5T0MO
   Cache Serial Number: PBKUD0BRH5T3I6
   RAID 6 (ADG) Status: Enabled
   Controller Status: OK
   Hardware Revision: B
   Firmware Version: 4.68
   Wait for Cache Room: Disabled
   Surface Analysis Inconsistency Notification: Disabled
   Post Prompt Timeout: 15 secs
   Cache Board Present: True
   Cache Status: OK
   Drive Write Cache: Disabled
   Total Cache Size: 2.0 GB
   Total Cache Memory Available: 1.8 GB
   No-Battery Write Cache: Disabled
   Cache Backup Power Source: Capacitors
   Battery/Capacitor Count: 1
   Battery/Capacitor Status: OK
   SATA NCQ Supported: True
   Spare Activation Mode: Activate on physical drive failure (default)
   Controller Temperature (C): 88
   Cache Module Temperature (C): 37
   Capacitor Temperature  (C): 21
   Number of Ports: 6 (2 Internal / 4 External )
   Driver Name: hpsa
   Driver Version: 3.4.4
   Driver Supports HP SSD Smart Path: True



   unassigned

      physicaldrive 5I:1:1
         Port: 5I
         Box: 1
         Bay: 1
         Status: OK
         Drive Type: Unassigned Drive
         Interface Type: SAS
         Size: 500 GB
         Native Block Size: 512
         Rotational Speed: 15000
         Firmware Revision: HPD6
         Serial Number: 6SL7G55D0000N4173JLT
         Model: HP      EF0600FARNA
         Current Temperature (C): 35
         Maximum Temperature (C): 43
         PHY Count: 2
         PHY Transfer Rate: 6.0Gbps, Unknown
         Drive Authentication Status: OK
         Carrier Application Version: 11
         Carrier Bootloader Version: 6

      physicaldrive 5I:1:2
         Port: 5I
         Box: 1
         Bay: 2
         Status: OK
         Drive Type: Unassigned Drive
         Interface Type: SAS
         Size: 600 GB
         Native Block Size: 512
         Rotational Speed: 15000
         Firmware Revision: HPD6
         Serial Number: 6SL7H2DM0000B41800Y0
         Model: HP      EF0600FARNA
         Current Temperature (C): 35
         Maximum Temperature (C): 44
         PHY Count: 2
         PHY Transfer Rate: 6.0Gbps, Unknown
         Drive Authentication Status: OK
         Carrier Application Version: 11
         Carrier Bootloader Version: 6

      physicaldrive 5I:1:3
         Port: 5I
         Box: 1
         Bay: 1
         Status: OK
         Drive Type: Unassigned Drive
         Interface Type: SAS
         Size: 700 GB
         Native Block Size: 512
         Rotational Speed: 15000
         Firmware Revision: HPD6
         Serial Number: 6SL7G55D0000N4173JLT
         Model: HP      EF0600FARNA
         Current Temperature (C): 35
         Maximum Temperature (C): 43
         PHY Count: 2
         PHY Transfer Rate: 6.0Gbps, Unknown
         Drive Authentication Status: OK
         Carrier Application Version: 11
         Carrier Bootloader Version: 6


   SEP (Vendor ID PMCSIERA, Model SRCv24x6G) 380
      Device Number: 380
      Firmware Version: RevB
      WWID: 5001438028842E1F
      Vendor ID: PMCSIERA
      Model: SRCv24x6G
'''

ONE_DRIVE_RAID_1 = '''

Smart Array P822 in Slot 2
   Bus Interface: PCI
   Slot: 2
   Serial Number: PDVTF0BRH5T0MO
   Cache Serial Number: PBKUD0BRH5T3I6
   RAID 6 (ADG) Status: Enabled
   Controller Status: OK
   Hardware Revision: B
   Firmware Version: 4.68
   Rebuild Priority: Medium
   Expand Priority: Medium
   Surface Scan Delay: 3 secs
   Surface Scan Mode: Idle
   Queue Depth: Automatic
   Monitor and Performance Delay: 60  min
   Elevator Sort: Enabled
   Degraded Performance Optimization: Disabled
   Inconsistency Repair Policy: Disabled
   Wait for Cache Room: Disabled
   Surface Analysis Inconsistency Notification: Disabled
   Post Prompt Timeout: 15 secs
   Cache Board Present: True
   Cache Status: OK
   Cache Ratio: 10% Read / 90% Write
   Drive Write Cache: Disabled
   Total Cache Size: 2.0 GB
   Total Cache Memory Available: 1.8 GB
   No-Battery Write Cache: Disabled
   Cache Backup Power Source: Capacitors
   Battery/Capacitor Count: 1
   Battery/Capacitor Status: OK
   SATA NCQ Supported: True
   Spare Activation Mode: Activate on physical drive failure (default)
   Controller Temperature (C): 88
   Cache Module Temperature (C): 38
   Capacitor Temperature  (C): 23
   Number of Ports: 6 (2 Internal / 4 External )
   Driver Name: hpsa
   Driver Version: 3.4.4
   Driver Supports HP SSD Smart Path: True

   Array: A
      Interface Type: SAS
      Unused Space: 1042189  MB
      Status: OK
      MultiDomain Status: OK
      Array Type: Data
      HP SSD Smart Path: disable



      Logical Drive: 1
         Size: 50.0 GB
         Fault Tolerance: 1
         Heads: 255
         Sectors Per Track: 32
         Cylinders: 12850
         Strip Size: 256 KB
         Full Stripe Size: 256 KB
         Status: OK
         MultiDomain Status: OK
         Caching:  Enabled
         Unique Identifier: 600508B1001C02BDBCB659B8A264186A
         Disk Name: /dev/sda
         Mount Points: None
         Logical Drive Label: 02896A0EPDVTF0BRH5T0MOEBAA
         Mirror Group 0:
            physicaldrive 5I:1:1 (port 5I:box 1:bay 1, SAS, 600 GB, OK)
         Mirror Group 1:
            physicaldrive 5I:1:2 (port 5I:box 1:bay 2, SAS, 600 GB, OK)
         Drive Type: Data
         LD Acceleration Method: Controller Cache

      physicaldrive 5I:1:1
         Port: 5I
         Box: 1
         Bay: 1
         Status: OK
         Drive Type: Data Drive
         Interface Type: SAS
         Size: 600 GB
         Native Block Size: 512
         Rotational Speed: 15000
         Firmware Revision: HPD5
         Serial Number: 6SL7G55D0000N4173JLT
         Model: HP      EF0600FARNA
         Current Temperature (C): 37
         Maximum Temperature (C): 43
         PHY Count: 2
         PHY Transfer Rate: 6.0Gbps, Unknown
         Drive Authentication Status: OK
         Carrier Application Version: 11
         Carrier Bootloader Version: 6

      physicaldrive 5I:1:2
         Port: 5I
         Box: 1
         Bay: 2
         Status: OK
         Drive Type: Data Drive
         Interface Type: SAS
         Size: 600 GB
         Native Block Size: 512
         Rotational Speed: 15000
         Firmware Revision: HPD6
         Serial Number: 6SL7H2DM0000B41800Y0
         Model: HP      EF0600FARNA
         Current Temperature (C): 37
         Maximum Temperature (C): 44
         PHY Count: 2
         PHY Transfer Rate: 6.0Gbps, Unknown
         Drive Authentication Status: OK
         Carrier Application Version: 11
         Carrier Bootloader Version: 6

   unassigned

      physicaldrive 5I:1:3
         Port: 5I
         Box: 1
         Bay: 1
         Status: OK
         Drive Type: Unassigned Drive
         Interface Type: SAS
         Size: 500 GB
         Native Block Size: 512
         Rotational Speed: 15000
         Firmware Revision: HPD6
         Serial Number: 6SL7G55D0000N4173JLT
         Model: HP      EF0600FARNA
         Current Temperature (C): 35
         Maximum Temperature (C): 43
         PHY Count: 2
         PHY Transfer Rate: 6.0Gbps, Unknown
         Drive Authentication Status: OK
         Carrier Application Version: 11
         Carrier Bootloader Version: 6

'''

DRIVE_2_RAID_1_OKAY_TO_SHARE = '''

Available options are:
   Max: 521094 (Units in MB)
   Min: 16 (Units in MB)



'''

TWO_DRIVES_50GB_RAID1 = '''

Smart Array P822 in Slot 2
   Bus Interface: PCI
   Slot: 2
   Serial Number: PDVTF0BRH5T0MO
   Cache Serial Number: PBKUD0BRH5T3I6
   RAID 6 (ADG) Status: Enabled
   Controller Status: OK
   Hardware Revision: B
   Firmware Version: 4.68
   Rebuild Priority: Medium
   Expand Priority: Medium
   Surface Scan Delay: 3 secs
   Surface Scan Mode: Idle
   Queue Depth: Automatic
   Monitor and Performance Delay: 60  min
   Elevator Sort: Enabled
   Degraded Performance Optimization: Disabled
   Inconsistency Repair Policy: Disabled
   Wait for Cache Room: Disabled
   Surface Analysis Inconsistency Notification: Disabled
   Post Prompt Timeout: 15 secs
   Cache Board Present: True
   Cache Status: OK
   Cache Ratio: 10% Read / 90% Write
   Drive Write Cache: Disabled
   Total Cache Size: 2.0 GB
   Total Cache Memory Available: 1.8 GB
   No-Battery Write Cache: Disabled
   Cache Backup Power Source: Capacitors
   Battery/Capacitor Count: 1
   Battery/Capacitor Status: OK
   SATA NCQ Supported: True
   Spare Activation Mode: Activate on physical drive failure (default)
   Controller Temperature (C): 88
   Cache Module Temperature (C): 38
   Capacitor Temperature  (C): 23
   Number of Ports: 6 (2 Internal / 4 External )
   Driver Name: hpsa
   Driver Version: 3.4.4
   Driver Supports HP SSD Smart Path: True

   Array: A
      Interface Type: SAS
      Unused Space: 939791  MB
      Status: OK
      MultiDomain Status: OK
      Array Type: Data
      HP SSD Smart Path: disable



      Logical Drive: 1
         Size: 50.0 GB
         Fault Tolerance: 1
         Heads: 255
         Sectors Per Track: 32
         Cylinders: 12850
         Strip Size: 256 KB
         Full Stripe Size: 256 KB
         Status: OK
         MultiDomain Status: OK
         Caching:  Enabled
         Unique Identifier: 600508B1001C02BDBCB659B8A264186A
         Disk Name: /dev/sda
         Mount Points: None
         Logical Drive Label: 02896A0EPDVTF0BRH5T0MOEBAA
         Mirror Group 0:
            physicaldrive 5I:1:1 (port 5I:box 1:bay 1, SAS, 600 GB, OK)
         Mirror Group 1:
            physicaldrive 5I:1:2 (port 5I:box 1:bay 2, SAS, 600 GB, OK)
         Drive Type: Data
         LD Acceleration Method: Controller Cache
      Logical Drive: 2
         Size: 50.0 GB
         Fault Tolerance: 1
         Heads: 255
         Sectors Per Track: 32
         Cylinders: 12850
         Strip Size: 256 KB
         Full Stripe Size: 256 KB
         Status: OK
         MultiDomain Status: OK
         Caching:  Enabled
         Unique Identifier: 600508B1001C1614116817E8A9DA1D2F
         Disk Name: /dev/sdb
         Mount Points: None
         Logical Drive Label: 06896EEAPDVTF0BRH5T0MO55C7
         Mirror Group 0:
            physicaldrive 5I:1:1 (port 5I:box 1:bay 1, SAS, 600 GB, OK)
         Mirror Group 1:
            physicaldrive 5I:1:2 (port 5I:box 1:bay 2, SAS, 600 GB, OK)
         Drive Type: Data
         LD Acceleration Method: Controller Cache

      physicaldrive 5I:1:1
         Port: 5I
         Box: 1
         Bay: 1
         Status: OK
         Drive Type: Data Drive
         Interface Type: SAS
         Size: 600 GB
         Native Block Size: 512
         Rotational Speed: 15000
         Firmware Revision: HPD6
         Serial Number: 6SL7G55D0000N4173JLT
         Model: HP      EF0600FARNA
         Current Temperature (C): 37
         Maximum Temperature (C): 43
         PHY Count: 2
         PHY Transfer Rate: 6.0Gbps, Unknown
         Drive Authentication Status: OK
         Carrier Application Version: 11
         Carrier Bootloader Version: 6

      physicaldrive 5I:1:2
         Port: 5I
         Box: 1
         Bay: 2
         Status: OK
         Drive Type: Data Drive
         Interface Type: SAS
         Size: 600 GB
         Native Block Size: 512
         Rotational Speed: 15000
         Firmware Revision: HPD6
         Serial Number: 6SL7H2DM0000B41800Y0
         Model: HP      EF0600FARNA
         Current Temperature (C): 37
         Maximum Temperature (C): 44
         PHY Count: 2
         PHY Transfer Rate: 6.0Gbps, Unknown
         Drive Authentication Status: OK
         Carrier Application Version: 11
         Carrier Bootloader Version: 6

   unassigned

      physicaldrive 5I:1:3
         Port: 5I
         Box: 1
         Bay: 2
         Status: OK
         Drive Type: Data Drive
         Interface Type: SAS
         Size: 600 GB
         Native Block Size: 512
         Rotational Speed: 15000
         Firmware Revision: HPD6
         Serial Number: 6SL7H2DM0000B41800Y0
         Model: HP      EF0600FARNA
         Current Temperature (C): 37
         Maximum Temperature (C): 44
         PHY Count: 2
         PHY Transfer Rate: 6.0Gbps, Unknown
         Drive Authentication Status: OK
         Carrier Application Version: 11
         Carrier Bootloader Version: 6


   SEP (Vendor ID PMCSIERA, Model SRCv24x6G) 380
      Device Number: 380
      Firmware Version: RevB
      WWID: 5001438028842E1F
      Vendor ID: PMCSIERA
      Model: SRCv24x6G
'''


NO_DRIVES_HPSSA_7_DISKS = '''

Smart Array P822 in Slot 3
   Bus Interface: PCI
   Slot: 3
   Serial Number: PDVTF0BRH5T0KV

   unassigned

      physicaldrive 5I:1:1
         Port: 5I
         Box: 1
         Bay: 1
         Status: OK
         Interface Type: SAS
         Size: 199 GB
         Firmware Revision: HPD6
         Serial Number: 6SL7G4QV0000B41803GZ
         Model: HP      EF0600FARNA

      physicaldrive 5I:1:2
         Port: 5I
         Box: 1
         Bay: 2
         Status: OK
         Interface Type: SAS
         Size: 200 GB
         Firmware Revision: HPD6
         Serial Number: 6SL7HK0Y0000N419008G
         Model: HP      EF0600FARNA

      physicaldrive 5I:1:3
         Port: 5I
         Box: 1
         Bay: 3
         Status: OK
         Interface Type: SAS
         Size: 600 GB
         Firmware Revision: HPD6
         Serial Number: 6SL7H1L50000B4180V5Y
         Model: HP      EF0600FARNA

      physicaldrive 5I:1:4
         Port: 5I
         Box: 1
         Bay: 4
         Status: OK
         Interface Type: SAS
         Size: 599 GB
         Firmware Revision: HPD6
         Serial Number: 6SL7H1K30000B41800TT
         Model: HP      EF0600FARNA

      physicaldrive 6I:1:5
         Port: 6I
         Box: 1
         Bay: 5
         Status: OK
         Interface Type: SAS
         Size: 598 GB
         Firmware Revision: HPDB
         Serial Number: 2AVUR97N
         Model: HP      EF0600FATFF

      physicaldrive 6I:1:6
         Port: 6I
         Box: 1
         Bay: 6
         Status: OK
         Interface Type: SAS
         Size: 500 GB
         Firmware Revision: HPDB
         Serial Number: 2AVVJR1N
         Model: HP      EF0600FATFF

      physicaldrive 6I:1:7
         Port: 6I
         Box: 1
         Bay: 7
         Status: OK
         Interface Type: SAS
         Size: 500 GB
         Firmware Revision: HPDB
         Serial Number: 2AVVENJN
         Model: HP      EF0600FATFF
'''


ONE_DRIVE_RAID_1_50_GB = '''

Smart Array P822 in Slot 3
   Slot: 3
   Serial Number: PDVTF0BRH5T0KV

   Array: A
      Interface Type: SAS
      Unused Space: 1042189  MB (91.1%)
      Used Space: 100.0 GB (8.9%)

      Logical Drive: 1
         Size: 50.0 GB
         Fault Tolerance: 1
         Status: OK
         MultiDomain Status: OK
         Unique Identifier: 600508B1001C861A72C774A7394AE2AC
         Disk Name: /dev/sda
         Logical Drive Label: 013400ABPDVTF0BRH5T0KV22C5
         LD Acceleration Method: Controller Cache

      physicaldrive 5I:1:1
         Port: 5I
         Box: 1
         Bay: 1
         Status: OK
         Interface Type: SAS
         Size: 199 GB
         Firmware Revision: HPD6
         Serial Number: 6SL7G4QV0000B41803GZ
         Model: HP      EF0600FARNA

      physicaldrive 5I:1:2
         Port: 5I
         Box: 1
         Bay: 2
         Status: OK
         Interface Type: SAS
         Size: 200 GB
         Firmware Revision: HPD6
         Serial Number: 6SL7HK0Y0000N419008G
         Model: HP      EF0600FARNA

   unassigned

      physicaldrive 5I:1:3
         Port: 5I
         Box: 1
         Bay: 3
         Status: OK
         Interface Type: SAS
         Size: 600 GB
         Firmware Revision: HPD6
         Serial Number: 6SL7H1L50000B4180V5Y
         Model: HP      EF0600FARNA

      physicaldrive 5I:1:4
         Port: 5I
         Box: 1
         Bay: 4
         Status: OK
         Interface Type: SAS
         Size: 599 GB
         Firmware Revision: HPD6
         Serial Number: 6SL7H1K30000B41800TT
         Model: HP      EF0600FARNA

      physicaldrive 6I:1:5
         Port: 6I
         Box: 1
         Bay: 5
         Status: OK
         Interface Type: SAS
         Size: 598 GB
         Firmware Revision: HPDB
         Serial Number: 2AVUR97N
         Model: HP      EF0600FATFF

      physicaldrive 6I:1:6
         Port: 6I
         Box: 1
         Bay: 6
         Status: OK
         Interface Type: SAS
         Size: 500 GB
         Firmware Revision: HPDB
         Serial Number: 2AVVJR1N
         Model: HP      EF0600FATFF

      physicaldrive 6I:1:7
         Port: 6I
         Box: 1
         Bay: 7
         Status: OK
         Interface Type: SAS
         Size: 500 GB
         Firmware Revision: HPDB
         Serial Number: 2AVVENJN
         Model: HP      EF0600FATFF
'''


TWO_DRIVES_50GB_RAID1_MAXGB_RAID5 = '''

Smart Array P822 in Slot 3
   Slot: 3
   Serial Number: PDVTF0BRH5T0KV

   Array: A
      Interface Type: SAS
      Unused Space: 1042189  MB (91.1%)
      Used Space: 100.0 GB (8.9%)
      Status: OK

      Logical Drive: 1
         Size: 50.0 GB
         Fault Tolerance: 1
         Status: OK
         Unique Identifier: 600508B1001C861A72C774A7394AE2AC
         Disk Name: /dev/sda

      physicaldrive 5I:1:1
         Port: 5I
         Box: 1
         Bay: 1
         Status: OK
         Interface Type: SAS
         Size: 199 GB
         Firmware Revision: HPD6
         Serial Number: 6SL7G4QV0000B41803GZ
         Model: HP      EF0600FARNA

      physicaldrive 5I:1:2
         Port: 5I
         Box: 1
         Bay: 2
         Status: OK
         Interface Type: SAS
         Size: 200 GB
         Firmware Revision: HPD6
         Serial Number: 6SL7HK0Y0000N419008G
         Model: HP      EF0600FARNA


   Array: B
      Interface Type: SAS
      Unused Space: 0  MB (0.0%)
      Used Space: 1.6 TB (100.0%)
      Status: OK
      MultiDomain Status: OK
      Array Type: Data
      HP SSD Smart Path: disable

      Logical Drive: 2
         Size: 1.1 TB
         Fault Tolerance: 5
         Status: OK
         Unique Identifier: 600508B1001CE9DE8551AEE29D5A72F7

      physicaldrive 5I:1:3
         Port: 5I
         Box: 1
         Bay: 3
         Status: OK
         Interface Type: SAS
         Size: 600 GB
         Firmware Revision: HPD6
         Serial Number: 6SL7H1L50000B4180V5Y
         Model: HP      EF0600FARNA

      physicaldrive 5I:1:4
         Port: 5I
         Box: 1
         Bay: 4
         Status: OK
         Interface Type: SAS
         Size: 599 GB
         Firmware Revision: HPD6
         Serial Number: 6SL7H1K30000B41800TT
         Model: HP      EF0600FARNA

      physicaldrive 6I:1:5
         Port: 6I
         Box: 1
         Bay: 5
         Status: OK
         Interface Type: SAS
         Size: 598 GB
         Firmware Revision: HPDB
         Serial Number: 2AVUR97N
         Model: HP      EF0600FATFF

   unassigned

      physicaldrive 6I:1:6
         Port: 6I
         Box: 1
         Bay: 6
         Status: OK
         Interface Type: SAS
         Size: 500 GB
         Firmware Revision: HPDB
         Serial Number: 2AVVJR1N

      physicaldrive 6I:1:7
         Port: 6I
         Box: 1
         Bay: 7
         Status: OK
         Interface Type: SAS
         Size: 500 GB
         Firmware Revision: HPDB
         Serial Number: 2AVVENJN
         Model: HP      EF0600FATFF
'''

HPSSA_HBA_MODE = '''

Smart Array P822 in Slot 3
   Bus Interface: PCI
   Slot: 3
   Serial Number: PDVTF0BRH5T0KV
   Cache Serial Number: PBKUD0BRH5T3UM
   RAID 6 (ADG) Status: Enabled
   Controller Status: OK
   Hardware Revision: B
   Firmware Version: 5.22
   Cache Board Present: True
   Cache Status: Not Configured
   Total Cache Size: 2.0 GB
   Total Cache Memory Available: 1.8 GB
   Cache Backup Power Source: Capacitors
   Battery/Capacitor Count: 1
   Battery/Capacitor Status: OK
   Controller Temperature (C): 88
   Cache Module Temperature (C): 37
   Capacitor Temperature  (C): 24
   Number of Ports: 6 (2 Internal / 4 External )
   Driver Name: hpsa
   Driver Version: 3.4.14
   HBA Mode Enabled: True
   PCI Address (Domain:Bus:Device.Function): 0000:0D:00.0
   Host Serial Number: SGH401AERD
   Sanitize Erase Supported: False
   Primary Boot Volume: None
   Secondary Boot Volume: None


   Port Name: 5I
         Port ID: 0
         Port Connection Number: 0
         SAS Address: 5001438028842E40
         Port Location: Internal

   Internal Drive Cage at Port 5I, Box 1, OK
      Power Supply Status: Not Redundant
      Drive Bays: 4
      Port: 5I
      Box: 1
      Location: Internal

   Physical Drives
      physicaldrive 5I:1:1 (port 5I:box 1:bay 1, SAS, 600 GB, OK)



   unassigned

      physicaldrive 5I:1:1
         Port: 5I
         Box: 1
         Bay: 1
         Status: OK
         Drive Type: HBA Mode Drive
         Interface Type: SAS
         Size: 600 GB
         Drive exposed to OS: True
         Native Block Size: 512
         Rotational Speed: 15000
         Firmware Revision: HPD6
         Serial Number: 6SL7G4QV0000B41803GZ
         Model: HP      EF0600FARNA
         Current Temperature (C): 36
         Maximum Temperature (C): 45
         PHY Count: 2
         PHY Transfer Rate: 6.0Gbps, Unknown
         Drive Authentication Status: OK
         Carrier Application Version: 11
         Carrier Bootloader Version: 6
         Disk Name: /dev/sda
         Mount Points: None
         Sanitize Erase Supported: False
'''

SSA_ERASE_DRIVE = '''

Smart Array P440 in Slot 2
   Bus Interface: PCI
   Slot: 2
   Serial Number: PDNMF0ARH8Y342
   RAID 6 (ADG) Status: Enabled
   Controller Status: OK
   Firmware Version: 4.52-0
   Spare Activation Mode: Activate on physical drive failure (default)
   Encryption: Disabled
   Driver Name: hpsa
   Driver Version: 3.4.16
   Controller Mode: RAID
   Pending Controller Mode: RAID
   Controller Mode Reboot: Not Required
   Host Serial Number: SGH537Y7AY
   Sanitize Erase Supported: True
   Primary Boot Volume: None
   Secondary Boot Volume: None


   Port Name: 1I
         Port ID: 0
         Port Connection Number: 0
         SAS Address: 5001438035544EC0
         Port Location: Internal
         Managed Cable Connected: False

   Physical Drives
      physicaldrive 1I:2:1 (port 1I:box 2:bay 1, SAS HDD, 300 GB, OK)

   unassigned

      physicaldrive 1I:2:1
         Port: 1I
         Box: 2
         Bay: 1
         Status: OK
         Drive Type: Unassigned Drive
         Interface Type: SAS
         Size: 300 GB
         Drive exposed to OS: False
         Logical/Physical Block Size: 512/512
         Rotational Speed: 15100
         Firmware Revision: HPD4
         Serial Number: S7K0C3FJ0000K601EZLM
         WWID: 5000C5008E183B1D
         Model: HP      EH0300JEDHC
         Current Temperature (C): 42
         Maximum Temperature (C): 52
         PHY Count: 2
         PHY Transfer Rate: 12.0Gbps, Unknown
         Drive Authentication Status: OK
         Carrier Application Version: 11
         Carrier Bootloader Version: 6
         Sanitize Erase Supported: True
         Sanitize Estimated Max Erase Time: 0 hour(s)36 minute(s)
         Unrestricted Sanitize Supported: False
         Shingled Magnetic Recording Support: None

      physicaldrive 6I:1:7
         Port: 6I
         Box: 1
         Bay: 7
         Status: OK
         Drive Type: Unassigned Drive
         Interface Type: Solid State SAS
         Size: 200 GB
         Native Block Size: 512
         Rotational Speed: 15000
         Firmware Revision: HPD6
         Serial Number: 6SL7G54Q0000N4180W34
         Model: HP      EF0600FARNA
         Current Temperature (C): 31
         Maximum Temperature (C): 39
         PHY Count: 2
         PHY Transfer Rate: 6.0Gbps, Unknown
         Drive Authentication Status: OK
         Carrier Application Version: 11
         Carrier Bootloader Version: 6
'''

SSA_ERASE_IN_PROGRESS = '''
Smart Array P440 in Slot 2
   Controller Mode: RAID
   Pending Controller Mode: RAID
   Sanitize Erase Supported: True
   Primary Boot Volume: None
   Secondary Boot Volume: None

   unassigned

      physicaldrive 1I:2:1
         Drive Type: Unassigned Drive
         Interface Type: SAS
         Size: 300 GB
         Status: Erase In Progress
         Drive Type: Unassigned Drive
         Sanitize Erase Supported: True
         Sanitize Estimated Max Erase Time: 0 hour(s)36 minute(s)
         Unrestricted Sanitize Supported: False
'''

SSA_ERASE_COMPLETE = '''
Smart Array P440 in Slot 2
   Controller Mode: RAID
   Pending Controller Mode: RAID
   Sanitize Erase Supported: True
   Primary Boot Volume: None
   Secondary Boot Volume: None

   unassigned

      physicaldrive 1I:2:1
         Drive Type: Unassigned Drive
         Interface Type: SAS
         Size: 300 GB
         Status: Erase Complete. Reenable Before Using.
         Drive Type: Unassigned Drive
         Sanitize Erase Supported: True
         Sanitize Estimated Max Erase Time: 0 hour(s)36 minute(s)
         Unrestricted Sanitize Supported: False
'''

SSA_ERASE_NOT_SUPPORTED = '''

Smart Array P440 in Slot 2
   Controller Status: OK
   Firmware Version: 4.52-0
   Spare Activation Mode: Activate on physical drive failure (default)
   Controller Mode: RAID
   Pending Controller Mode: RAID
   Controller Mode Reboot: Not Required
   Sanitize Erase Supported: False
   Primary Boot Volume: None
   Secondary Boot Volume: None

   unassigned

      physicaldrive 1I:2:1
         Drive Type: Unassigned Drive
         Interface Type: SAS
         Size: 300 GB
         Status: OK
         Drive Type: Unassigned Drive
         Sanitize Erase Supported: False
         Sanitize Estimated Max Erase Time: 0 hour(s)36 minute(s)
         Unrestricted Sanitize Supported: False
'''

SSA_ERASE_COMPLETE_NOT_SUPPORTED = '''

Smart Array P440 in Slot 2
   Controller Status: OK
   Firmware Version: 4.52-0
   Spare Activation Mode: Activate on physical drive failure (default)
   Controller Mode: RAID
   Pending Controller Mode: RAID
   Controller Mode Reboot: Not Required
   Sanitize Erase Supported: False
   Primary Boot Volume: None
   Secondary Boot Volume: None

   unassigned

      physicaldrive 1I:2:1
         Drive Type: Unassigned Drive
         Interface Type: SAS
         Size: 300 GB
         Status: Erase Complete. Reenable Before Using.
         Drive Type: Unassigned Drive
         Sanitize Erase Supported: False
         Sanitize Estimated Max Erase Time: 0 hour(s)36 minute(s)
         Unrestricted Sanitize Supported: False
'''

SSA_ERASE_IN_PROGRESS_NOT_SUPPORTED = '''
Smart Array P440 in Slot 2
   Controller Mode: RAID
   Pending Controller Mode: RAID
   Sanitize Erase Supported: True
   Primary Boot Volume: None
   Secondary Boot Volume: None

   unassigned

      physicaldrive 1I:2:1
         Drive Type: Unassigned Drive
         Interface Type: SAS
         Size: 300 GB
         Status: Erase In Progress
         Drive Type: Unassigned Drive
         Sanitize Erase Supported: False
         Sanitize Estimated Max Erase Time: 0 hour(s)36 minute(s)
         Unrestricted Sanitize Supported: False
'''

SSACLI_PARSING_TESTS = '''
Smart HBA H240 in Slot 1 (RAID Mode)
   Slot: 1
   Controller Mode: RAID Mode

   Internal Drive Cage at Port 1I, Box 1, OK
      Drive Bays: 4
      Port: 1I
      Box: 1

   Physical Drives
      physicaldrive 1I:1:4 (port 1I:box 1:bay 4, SAS HDD, 900 GB, OK)
      physicaldrive 1I:1:3 (port 1I:box 1:bay 3, SAS HDD, 900 GB, OK)

   Internal Drive Cage at Port 2I, Box 1, OK
      Drive Bays: 4
      Port: 2I
      Box: 1

   Physical Drives
      physicaldrive 2I:1:5 (port 2I:box 1:bay 5, SAS HDD, 900 GB, OK)
      physicaldrive 2I:1:6 (port 2I:box 1:bay 6, SAS HDD, 900 GB, OK)

   Unassigned
      physicaldrive 1I:1:4
         Port: 1I
         Box: 1
         Bay: 4
         Size: 900 GB
         Interface Type: SAS

Smart HBA H240 in Slot 2 (RAID Mode)
   Slot: 2
   Controller Mode: RAID Mode
   PCI Address (Domain:Bus:Device.Function): 0000:0B:00.0

   Array: H
      Interface Type: SAS

      Logical Drive: 8
         Size: 838.3 GB
         Status: OK

      physicaldrive 2I:2:8
         Port: 2I
         Box: 2
         Bay: 8
         Size: 900 GB
         Interface Type: SAS

Smart HBA H240 in Slot 3 (RAID Mode)
   Slot: 3
   Controller Mode: RAID Mode

Smart HBA H240ar in Slot 0 (Embedded) (RAID Mode)
   Bus Interface: PCI
   Slot: 0
'''



# Copyright 2020 Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""Test Class for SecurityParamsTestCase"""

import json
from unittest import mock

import testtools

from proliantutils.redfish.resources.manager import security_params


class SecurityParamsTestCase(testtools.TestCase):

    def setUp(self):
        super(SecurityParamsTestCase, self).setUp()
        self.conn = mock.Mock()
        security_param_file = ('proliantutils/tests/redfish/json_samples/'
                               'security_param.json')
        with open(security_param_file) as f:
            self.json_doc = json.load(f)
            self.conn.get.return_value.json.return_value = (
                self.json_doc)

        path = ("/redfish/v1/Mangers/1/SecurityService/"
                "SecurityDashboard/SecurityParams")
        self.sec_param = security_params.SecurityParams(
            self.conn, path, redfish_version='1.0.2',
            registries=None, root=None)

    def test__parse_attributes(self):
        self.sec_param._parse_attributes(self.json_doc)
        self.assertEqual('1.0.2', self.sec_param.redfish_version)
        self.assertEqual("Security Override Switch", self.sec_param.name)
        self.assertEqual("Ok", self.sec_param.status)
        self.assertEqual("Off", self.sec_param.state)

    def test_update_security_param_ignore_status(self):
        self.sec_param.update_security_param_ignore_status(ignore=False)
        data = {"Ignore": False}
        target_uri = ('/redfish/v1/Mangers/1/SecurityService/'
                      'SecurityDashboard/SecurityParams')
        self.sec_param._conn.patch.assert_called_once_with(
            target_uri, data=data)


class SecurityParamsCollectionTestCase(testtools.TestCase):

    def setUp(self):
        super(SecurityParamsCollectionTestCase, self).setUp()
        self.conn = mock.Mock()
        with open('proliantutils/tests/redfish/json_samples/'
                  'security_params_collection.json', 'r') as f:
            self.json_doc = json.load(f)
            self.conn.get.return_value.json.return_value = self.json_doc
        self.sec_params_col = security_params.SecurityParamsCollection(
            self.conn,
            ('/redfish/v1/Managers/1/SecurityService/'
             'SecurityDashboard/SecurityParams'),
            redfish_version='1.0.2', registries=None, root=None)

    def test__parse_attributes(self):
        self.sec_params_col._parse_attributes(self.json_doc)
        self.assertEqual('1.0.2', self.sec_params_col.redfish_version)
        self.assertEqual('Security Parameter Collection',
                         self.sec_params_col.name)
        path = ('/redfish/v1/Managers/1/SecurityService/'
                'SecurityDashboard/SecurityParams/0',
                '/redfish/v1/Managers/1/SecurityService/'
                'SecurityDashboard/SecurityParams/1')
        self.assertEqual(path, self.sec_params_col.members_identities)

    @mock.patch.object(security_params, 'SecurityParams', autospec=True)
    def test_get_member(self, mock_eth):
        self.sec_params_col.get_member(
            '/redfish/v1/Managers/1/SecurityService/SecurityDashboard/'
            'SecurityParams/1')
        mock_eth.assert_called_once_with(
            self.sec_params_col._conn,
            ('/redfish/v1/Managers/1/SecurityService/SecurityDashboard/'
             'SecurityParams/1'),
            redfish_version=self.sec_params_col.redfish_version,
            registries=None, root=self.sec_params_col.root)

    @mock.patch.object(security_params, 'SecurityParams', autospec=True)
    def test_get_members(self, mock_eth):
        members = self.sec_params_col.get_members()
        path = ('/redfish/v1/Managers/1/SecurityService/SecurityDashboard/'
                'SecurityParams/0')
        path2 = ('/redfish/v1/Managers/1/SecurityService/SecurityDashboard/'
                 'SecurityParams/1')
        calls = [mock.call(self.sec_params_col._conn, path,
                           redfish_version=self.sec_params_col.redfish_version,
                           registries=None, root=self.sec_params_col.root),
                 mock.call(self.sec_params_col._conn, path2,
                           redfish_version=self.sec_params_col.redfish_version,
                           registries=None, root=self.sec_params_col.root)]
        mock_eth.assert_has_calls(calls)
        self.assertIsInstance(members, list)
        self.assertEqual(2, len(members))



# Copyright 2017 Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

__author__ = 'HPE'

from sushy import exceptions as sushy_exceptions
from sushy.resources import base
from sushy.resources import common
from sushy.resources.manager import virtual_media

from proliantutils import exception
from proliantutils import log
from proliantutils.redfish.resources.manager import mapping as maps


LOG = log.get_logger(__name__)


def _get_media(media_types):
    """Helper method to map the media types."""
    get_mapped_media = (lambda x: maps.VIRTUAL_MEDIA_TYPES_MAP[x]
                        if x in maps.VIRTUAL_MEDIA_TYPES_MAP else None)
    return list(map(get_mapped_media, media_types))


class ActionsField(base.CompositeField):
    insert_vmedia = common.ResetActionField(
        '#HpeiLOVirtualMedia.InsertVirtualMedia')

    eject_vmedia = common.ResetActionField(
        '#HpeiLOVirtualMedia.EjectVirtualMedia')


class VirtualMedia(virtual_media.VirtualMedia):

    media_types = base.Field('MediaTypes', adapter=_get_media,
                             required=True)
    """A list of allowed media types for the instance."""

    _hpe_actions = ActionsField(['Oem', 'Hpe', 'Actions'], required=True)

    def _get_action_element(self, action_type):
        """Helper method to return the action object."""
        action = eval("self._hpe_actions." + action_type + "_vmedia")

        if not action:
            if action_type == "insert":
                action_path = '#HpeiLOVirtualMedia.InsertVirtualMedia'
            else:
                action_path = '#HpeiLOVirtualMedia.EjectVirtualMedia'

            raise exception.MissingAttributeError(
                attribute=action_path,
                resource=self._path)

        return action

    def insert_media(self, url):
        """Inserts Virtual Media to the device

        :param url: URL to image.
        :raises: SushyError, on an error from iLO.
        """
        try:
            super(VirtualMedia, self).insert_media(url, write_protected=True)
        except sushy_exceptions.SushyError:
            target_uri = self._get_action_element('insert').target_uri
            data = {'Image': url}
            self._conn.post(target_uri, data=data)

    def eject_media(self):
        """Ejects Virtual Media.

        :raises: SushyError, on an error from iLO.
        """
        try:
            super(VirtualMedia, self).eject_media()
        except sushy_exceptions.SushyError:
            target_uri = self._get_action_element('eject').target_uri
            self._conn.post(target_uri, data={})

    def set_vm_status(self, boot_on_next_reset):
        """Set the Virtual Media drive status.

        :param boot_on_next_reset: boolean value
        :raises: SushyError, on an error from iLO.
        """
        data = {
            "Oem": {
                "Hpe": {
                    "BootOnNextServerReset": boot_on_next_reset
                }
            }
        }
        self._conn.patch(self.path, data=data)


class VirtualMediaCollection(base.ResourceCollectionBase):

    @property
    def _resource_type(self):
        return VirtualMedia

    def get_member_device(self, device):
        """Returns the given virtual media device object.

        :param  device: virtual media device to be queried
        :returns virtual media device object.
        """
        for vmedia_device in self.get_members():
            if device in vmedia_device.media_types:
                return vmedia_device



# Copyright 2017 Hewlett Packard Enterprise Development LP
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json
from unittest import mock

import testtools

from proliantutils.redfish.resources.system.storage import storage


class StorageTestCase(testtools.TestCase):

    def setUp(self):
        super(StorageTestCase, self).setUp()
        self.conn = mock.Mock()
        storage_file = 'proliantutils/tests/redfish/json_samples/storage.json'
        with open(storage_file) as f:
            self.json_doc = json.load(f)
        self.conn.get.return_value.json.return_value = self.json_doc

        path = ("/redfish/v1/Systems/437XR1138R2/Storage/1")
        self.sys_stor = storage.Storage(
            self.conn, path, redfish_version='1.0.2',
            registries=None, root=None)

    def test__parse_attributes(self):
        self.sys_stor._parse_attributes(self.json_doc)
        self.assertEqual('1.0.2', self.sys_stor.redfish_version)
        self.assertEqual('1', self.sys_stor.identity)
        self.assertEqual('Local Storage Controller', self.sys_stor.name)
        self.assertEqual('Integrated RAID Controller',
                         self.sys_stor.description)
        self.assertEqual(self.json_doc.get('Drives'),
                         self.sys_stor.drives)

    def test_volumes(self):
        log_coll = None
        log_dr = None
        self.conn.get.return_value.json.reset_mock()
        with open('proliantutils/tests/redfish/'
                  'json_samples/volume_collection.json') as f:
            log_coll = json.loads(f.read())
        with open('proliantutils/tests/redfish/'
                  'json_samples/volume.json') as f:
            log_dr = json.loads(f.read())
        self.conn.get.return_value.json.side_effect = [log_coll, log_dr]
        actual_volumes = self.sys_stor.volumes
        self.assertIs(actual_volumes,
                      self.sys_stor.volumes)
        self.sys_stor.invalidate()
        self.sys_stor.refresh(force=False)
        self.assertTrue(actual_volumes._is_stale)

    def test__drives_list(self):
        self.conn.get.return_value.json.reset_mock()
        with open('proliantutils/tests/redfish/'
                  'json_samples/drive.json') as f:
            dr_json = json.loads(f.read())
        self.conn.get.return_value.json.side_effect = [dr_json['drive1'],
                                                       dr_json['drive2'],
                                                       dr_json['drive3']]
        actual_dr = self.sys_stor._drives_list()
        self.assertIsInstance(actual_dr, list)

    def test_drives_maximum_size_bytes(self):
        self.conn.get.return_value.json.reset_mock()
        val = []
        path = ('proliantutils/tests/redfish/json_samples/'
                'drive.json')
        with open(path, 'r') as f:
            dr_json = json.loads(f.read())
            val.append(dr_json['drive1'])
            val.append(dr_json['drive2'])
            val.append(dr_json['drive3'])
            self.conn.get.return_value.json.side_effect = val
        expected = 899527000000
        actual = self.sys_stor.drives_maximum_size_bytes
        self.assertEqual(expected, actual)

    def test_has_ssd_true(self):
        self.conn.get.return_value.json.reset_mock()
        val = []
        path = ('proliantutils/tests/redfish/json_samples/'
                'drive.json')
        with open(path, 'r') as f:
            dr_json = json.loads(f.read())
            val.append(dr_json['drive1'])
            val.append(dr_json['drive2'])
            val.append(dr_json['drive3'])
            self.conn.get.return_value.json.side_effect = val
        self.assertTrue(self.sys_stor.has_ssd)

    def test_has_rotational(self):
        self.conn.get.return_value.json.reset_mock()
        val = []
        path = ('proliantutils/tests/redfish/json_samples/'
                'drive.json')
        with open(path, 'r') as f:
            dr_json = json.loads(f.read())
            val.append(dr_json['drive1'])
            val.append(dr_json['drive2'])
            val.append(dr_json['drive3'])
            self.conn.get.return_value.json.side_effect = val
        self.assertTrue(self.sys_stor.has_rotational)

    def test_has_nvme_ssd(self):
        self.conn.get.return_value.json.reset_mock()
        val = []
        path = ('proliantutils/tests/redfish/json_samples/'
                'drive.json')
        with open(path, 'r') as f:
            dr_json = json.loads(f.read())
            val.append(dr_json['drive1'])
            val.append(dr_json['drive2'])
            val.append(dr_json['drive3'])
            self.conn.get.return_value.json.side_effect = val
        self.assertTrue(self.sys_stor.has_nvme_ssd)

    def test_drive_rotational_speed_rpm(self):
        self.conn.get.return_value.json.reset_mock()
        val = []
        path = ('proliantutils/tests/redfish/json_samples/'
                'drive.json')
        with open(path, 'r') as f:
            dr_json = json.loads(f.read())
            val.append(dr_json['drive1'])
            val.append(dr_json['drive2'])
            val.append(dr_json['drive3'])
            self.conn.get.return_value.json.side_effect = val
        expected = set([15000, 10000])
        self.assertEqual(expected,
                         self.sys_stor.drive_rotational_speed_rpm)


class StorageCollectionTestCase(testtools.TestCase):

    def setUp(self):
        super(StorageCollectionTestCase, self).setUp()
        self.conn = mock.Mock()
        with open('proliantutils/tests/redfish/json_samples/'
                  'storage_collection.json') as f:
            self.json_doc = json.load(f)
        self.conn.get.return_value.json.return_value = self.json_doc
        self.sys_stor_col = storage.StorageCollection(
            self.conn, '/redfish/v1/Systems/437XR1138R2/Storage',
            redfish_version='1.0.2', registries=None, root=None)

    def test__parse_attributes(self):
        self.sys_stor_col._parse_attributes(self.json_doc)
        self.assertEqual('1.0.2', self.sys_stor_col.redfish_version)
        self.assertEqual('Storage Collection',
                         self.sys_stor_col.name)
        path = ('/redfish/v1/Systems/437XR1138R2/Storage/1',)
        self.assertEqual(path, self.sys_stor_col.members_identities)

    @mock.patch.object(storage, 'Storage', autospec=True)
    def test_get_member(self, mock_eth):
        self.sys_stor_col.get_member(
            '/redfish/v1/Systems/437XR1138R2/Storage/1')
        mock_eth.assert_called_once_with(
            self.sys_stor_col._conn,
            ('/redfish/v1/Systems/437XR1138R2/Storage/1'),
            redfish_version=self.sys_stor_col.redfish_version,
            registries=None, root=self.sys_stor_col.root)

    @mock.patch.object(storage, 'Storage', autospec=True)
    def test_get_members(self, mock_eth):
        members = self.sys_stor_col.get_members()
        path = ("/redfish/v1/Systems/437XR1138R2/Storage/1")
        calls = [
            mock.call(self.sys_stor_col._conn, path,
                      redfish_version=self.sys_stor_col.redfish_version,
                      registries=None, root=self.sys_stor_col.root),
        ]
        mock_eth.assert_has_calls(calls)
        self.assertIsInstance(members, list)
        self.assertEqual(1, len(members))

    def test_volumes_maximum_size_bytes(self):
        self.conn.get.return_value.json.reset_mock()
        val = []
        path = ('proliantutils/tests/redfish/json_samples/'
                'storage.json')
        with open(path, 'r') as f:
            val.append(json.loads(f.read()))
        path = ('proliantutils/tests/redfish/json_samples/'
                'volume_collection.json')
        with open(path, 'r') as f:
            val.append(json.loads(f.read()))
        path = ('proliantutils/tests/redfish/json_samples/'
                'volume.json')
        with open(path, 'r') as f:
            val.append(json.loads(f.read()))
            self.conn.get.return_value.json.side_effect = val
        expected = 899527000000
        actual = self.sys_stor_col.volumes_maximum_size_bytes
        self.assertEqual(expected, actual)

    def test_drives_maximum_size_bytes(self):
        self.conn.get.return_value.json.reset_mock()
        val = []
        path = ('proliantutils/tests/redfish/json_samples/'
                'storage.json')
        with open(path, 'r') as f:
            val.append(json.loads(f.read()))
        path = ('proliantutils/tests/redfish/json_samples/'
                'drive.json')
        with open(path, 'r') as f:
            dr_json = json.loads(f.read())
            val.append(dr_json['drive1'])
            val.append(dr_json['drive2'])
            val.append(dr_json['drive3'])
            self.conn.get.return_value.json.side_effect = val
        expected = 899527000000
        actual = self.sys_stor_col.drives_maximum_size_bytes
        self.assertEqual(expected, actual)

    def test_has_ssd_true(self):
        self.conn.get.return_value.json.reset_mock()
        val = []
        path = ('proliantutils/tests/redfish/json_samples/'
                'storage.json')
        with open(path, 'r') as f:
            val.append(json.loads(f.read()))
        path = ('proliantutils/tests/redfish/json_samples/'
                'drive.json')
        with open(path, 'r') as f:
            dr_json = json.loads(f.read())
            val.append(dr_json['drive1'])
            val.append(dr_json['drive2'])
            val.append(dr_json['drive3'])
            self.conn.get.return_value.json.side_effect = val
        self.assertTrue(self.sys_stor_col.has_ssd)

    def test_has_rotational(self):
        self.conn.get.return_value.json.reset_mock()
        val = []
        path = ('proliantutils/tests/redfish/json_samples/'
                'storage.json')
        with open(path, 'r') as f:
            val.append(json.loads(f.read()))
        path = ('proliantutils/tests/redfish/json_samples/'
                'drive.json')
        with open(path, 'r') as f:
            dr_json = json.loads(f.read())
            val.append(dr_json['drive1'])
            val.append(dr_json['drive2'])
            val.append(dr_json['drive3'])
            self.conn.get.return_value.json.side_effect = val
        self.assertTrue(self.sys_stor_col.has_rotational)

    def test_has_nvme_ssd(self):
        self.conn.get.return_value.json.reset_mock()
        val = []
        path = ('proliantutils/tests/redfish/json_samples/'
                'storage.json')
        with open(path, 'r') as f:
            val.append(json.loads(f.read()))
        path = ('proliantutils/tests/redfish/json_samples/'
                'drive.json')
        with open(path, 'r') as f:
            dr_json = json.loads(f.read())
            val.append(dr_json['drive1'])
            val.append(dr_json['drive2'])
            val.append(dr_json['drive3'])
            self.conn.get.return_value.json.side_effect = val
        self.assertTrue(self.sys_stor_col.has_nvme_ssd)

    def test_drive_rotational_speed_rpm(self):
        self.conn.get.return_value.json.reset_mock()
        val = []
        path = ('proliantutils/tests/redfish/json_samples/'
                'storage.json')
        with open(path, 'r') as f:
            val.append(json.loads(f.read()))
        path = ('proliantutils/tests/redfish/json_samples/'
                'drive.json')
        with open(path, 'r') as f:
            dr_json = json.loads(f.read())
            val.append(dr_json['drive1'])
            val.append(dr_json['drive2'])
            val.append(dr_json['drive3'])
            self.conn.get.return_value.json.side_effect = val
        expected = set([15000, 10000])
        self.assertEqual(expected,
                         self.sys_stor_col.drive_rotational_speed_rpm)



# Copyright 2022 Hewlett Packard Enterprise Development LP
# Copyright 2014 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""Exception Class for proliantutils module."""


class ProliantUtilsException(Exception):
    """Parent class for all Proliantutils exceptions."""
    pass


class InvalidInputError(Exception):

    message = "Invalid Input: %(reason)s"

    # Note(deray): Not mandating the user to provide the ``reason`` attribute
    # parameter while raising InvalidInputError exception. This is because of
    # backward-compatibility reasons. See its unit test file to know about its
    # different ways of usage.
    def __init__(self, message=None, **kwargs):

        if not message:
            message = self.message

            if 'reason' not in kwargs:
                kwargs['reason'] = 'Unknown'

        message = message % kwargs
        super(InvalidInputError, self).__init__(message)


class IloError(ProliantUtilsException):
    """Base Exception.

    This exception is used when a problem is encountered in
    executing an operation on the iLO.
    """
    def __init__(self, message, errorcode=None):
        super(IloError, self).__init__(message)


class IloClientInternalError(IloError):
    """Internal Error from IloClient.

    This exception is raised when iLO client library fails to
    communicate properly with the iLO.
    """
    def __init__(self, message, errorcode=None):
        super(IloClientInternalError, self).__init__(message)


class IloCommandNotSupportedError(IloError):
    """Command not supported on the platform.

    This exception is raised when iLO client library fails to
    communicate properly with the iLO
    """
    def __init__(self, message, errorcode=None):
        super(IloCommandNotSupportedError, self).__init__(message)


class IloCommandNotSupportedInBiosError(IloCommandNotSupportedError):
    """Command not supported on the bios boot mode.

    This exception is raised when iLO client library fails to
    communicate properly with the iLO
    """
    def __init__(self, message, errorcode=None):
        super(IloCommandNotSupportedInBiosError, self).__init__(message)


class IloLogicalDriveNotFoundError(IloError):
    """Logical drive not found error.

    This exception is raised when iLO client library unable to find
    any logical drive on storage controller
    """
    def __init__(self, message, errorcode=None):
        super(IloLogicalDriveNotFoundError, self).__init__(message)


class IloLoginFailError(IloError):
    """iLO Login Failed.

    This exception is used to communicate a login failure to
    the caller.
    """
    messages = ['User login name was not found',
                'Login failed', 'Login credentials rejected']
    statuses = [0x005f, 0x000a]
    message = 'Authorization Failed'

    def __init__(self, message, errorcode=None):
        super(IloLoginFailError, self).__init__(message)


class IloConnectionError(IloError):
    """Cannot connect to iLO.

    This exception is used to communicate an HTTP connection
    error from the iLO to the caller.
    """
    def __init__(self, message):
        super(IloConnectionError, self).__init__(message)

# This is not merged with generic InvalidInputError because
# of backward-compatibility reasons. If we changed this,
# use-cases of excepting 'IloError' to catch 'IloInvalidInputError'
# will be broken.


class IloInvalidInputError(IloError):
    """Invalid Input passed.

    This exception is used when invalid inputs are passed to
    the APIs exposed by this module.
    """
    def __init__(self, message):
        super(IloInvalidInputError, self).__init__(message)


class HPSSAException(ProliantUtilsException):

    message = "An exception occured in ssa module"

    def __init__(self, message=None, **kwargs):
        if not message:
            message = self.message

        message = message % kwargs
        super(HPSSAException, self).__init__(message)


class PhysicalDisksNotFoundError(HPSSAException):

    message = ("Not enough physical disks were found to create logical disk "
               "of size %(size_gb)s GB and raid level %(raid_level)s")


class HPSSAOperationError(HPSSAException):

    message = ("An error was encountered while doing ssa configuration: "
               "%(reason)s.")


class ImageExtractionFailed(ProliantUtilsException):
    message = "Failed to extract image %(image_ref)s, reason: %(reason)s"

    def __init__(self, message=None, **kwargs):
        if not message:
            message = self.message % kwargs

        super(ImageExtractionFailed, self).__init__(message)


class IloSNMPInvalidInputFailure(IloError):
    message = "Failed to do SNMP retreival %(reason)s"

    def __init__(self, message=None, **kwargs):
        if not message:
            message = self.message % kwargs

        super(IloSNMPInvalidInputFailure, self).__init__(message)


class IloSNMPExceptionFailure(IloError):
    message = "SNMP library failed %(reason)s"

    def __init__(self, message=None, **kwargs):
        if not message:
            message = self.message % kwargs

        super(IloSNMPExceptionFailure, self).__init__(message)


class ImageRefValidationFailed(ProliantUtilsException):
    message = ("Validation of image href %(image_href)s failed, "
               "reason: %(reason)s")

    def __init__(self, message=None, **kwargs):
        if not message:
            message = self.message % kwargs

        super(ImageRefValidationFailed, self).__init__(message)


class SUMOperationError(ProliantUtilsException):
    """SUM based firmware update operation error.

    This exception is used when a problem is encountered in
    executing a SUM operation.
    """

    message = ("An error occurred while performing SUM based firmware "
               "update, reason: %(reason)s")

    def __init__(self, message=None, **kwargs):
        if not message:
            message = self.message % kwargs

        super(SUMOperationError, self).__init__(message)


class RedfishError(ProliantUtilsException):
    """Basic exception for errors raised by Redfish operations."""

    message = None

    def __init__(self, **kwargs):
        if self.message and kwargs:
            self.message = self.message % kwargs

        super(RedfishError, self).__init__(self.message)


class MissingAttributeError(RedfishError):
    message = ('The attribute %(attribute)s is missing from the '
               'resource %(resource)s')


class InvalidParameterValueError(RedfishError):
    message = ('The parameter "%(parameter)s" value "%(value)s" is invalid. '
               'Valid values are: %(valid_values)s')


class CertificateCreationError(ProliantUtilsException):
    message = ("Failed to create HTTPS certificate"
               "reason: %(reason)s")

    def __init__(self, message=None, **kwargs):
        if not message:
            message = self.message % kwargs

        super(CertificateCreationError, self).__init__(message)



# Copyright 2021-2022 Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

__author__ = 'HPE'

from sushy.resources import base
from sushy import utils as sushy_utils

from proliantutils.redfish.resources.manager import security_params


class SecurityDashboard(base.ResourceBase):

    identity = base.Field('Id', required=True)
    """The identity for the instance."""

    overall_status = base.Field('OverallSecurityStatus', required=True)
    """Overall security status of the server"""

    server_configuration_lock_status = (
        base.Field('ServerConfigurationLockStatus'))

    security_params_collection_uri = (
        base.Field(["SecurityParameters", "@odata.id"], required=True))

    @property
    @sushy_utils.cache_it
    def securityparamscollectionuri(self):
        """Gets the list of instances for security params

        :returns: the list of instances of security params.
        """
        return security_params.SecurityParamsCollection(
            self._conn, self.security_params_collection_uri,
            redfish_version=self.redfish_version)



# Copyright 2017 Hewlett Packard Enterprise Development LP
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

# SUPPORTED_BOOT_MODE constants

SUPPORTED_BOOT_MODE_LEGACY_BIOS_ONLY = 'legacy bios only'
SUPPORTED_BOOT_MODE_UEFI_ONLY = 'uefi only'
SUPPORTED_BOOT_MODE_LEGACY_BIOS_AND_UEFI = 'legacy bios and uefi'
SUPPORTED_BIOS_PROPERTIES = [
    "AdvancedMemProtection",
    "AutoPowerOn",
    "BootMode",
    "BootOrderPolicy",
    "CollabPowerControl",
    "DynamicPowerCapping",
    "DynamicPowerResponse",
    "IntelligentProvisioning",
    "IntelPerfMonitoring",
    "IntelProcVtd",
    "IntelQpiFreq",
    "IntelTxt",
    "PowerProfile",
    "PowerRegulator",
    "ProcAes",
    "ProcCoreDisable",
    "ProcHyperthreading",
    "ProcNoExecute",
    "ProcTurbo",
    "ProcVirtualization",
    "SecureBootStatus",
    "Sriov",
    "ThermalConfig",
    "ThermalShutdown",
    "TpmState",
    "TpmType",
    "UefiOptimizedBoot"
]
SUPPORTED_REDFISH_BIOS_PROPERTIES = SUPPORTED_BIOS_PROPERTIES + [
    "WorkloadProfile"
]
